# -*- utf-8 -*-
from abc import ABC
from xml.dom.minidom import Element

from aestate.exception import TagHandlerError
from aestate.util.Log import ALog


class NodeHandler(ABC):
    """节点事件抽象"""

    # 操作
    def handleNode(self, target_obj): ...


class IfHandler(NodeHandler):
    """if标签事件"""

    def __init__(self, initial_field, field, params, value, symbol):
        self.initial_field = initial_field
        self.field = field
        self.params = params
        self.value = value
        self.symbol = symbol

    def parse_node(self):
        pass

    def handleNode(self, target_obj):
        if self.initial_field != self.field:
            # 转换成同类型
            value = type(self.params[self.field])(self.value)
            if self.symbol == '>=':
                success = self.params[self.field] >= value
            elif self.symbol == '<=':
                success = self.params[self.field] <= value
            elif self.symbol == '==':
                success = self.params[self.field] == value
            elif self.symbol == '!=':
                success = self.params[self.field] != value
            elif self.symbol == '>':
                success = self.params[self.field] > value
            elif self.symbol == '<':
                success = self.params[self.field] < value
            else:
                ALog.log_error(
                    msg=f'The node rule parsing failed and did not conform to the grammatical structure.{self.symbol}',
                    obj=TagHandlerError, LogObject=target_obj.log_obj, raise_exception=True)
                success = False
        else:
            # UPDATE 1.0.6a2 如果匹配不到key就设置为None
            if self.symbol != '!=':
                ALog.log_error(
                    msg=f'`None` cannot judge `value`.wrong compiled field:`{self.field}{self.symbol}{self.value}`',
                    obj=TagHandlerError, LogObject=target_obj.log_obj, raise_exception=True)
                success = False
            else:
                #
                success = self.value == 'None' \
                          or self.value == 'false' \
                          or self.value == 'False' \
                          or self.value == '' \
                          or self.value == 0
            # if self.symbol == '>=':
            #     success = self.field >= self.value
            # elif self.symbol == '<=':
            #     success = self.field <= self.value
            # elif self.symbol == '==':
            #     success = self.field == self.value
            # elif self.symbol == '!=':
            #     success = self.field != self.value
            # elif self.symbol == '>':
            #     success = self.field > self.value
            # elif self.symbol == '<':
            #     success = self.field < self.value
            # else:
            #     ALog.log_error(
            #         msg=f'The node rule parsing failed and did not conform to the grammatical structure.{self.symbol}',
            #         obj=TagHandlerError, LogObject=target_obj.log_obj, raise_exception=True)
            #     success = False
        return success

    @staticmethod
    def checking_mark(node: Element):
        if node.nextSibling:
            if node.nextSibling.nodeName == '#text':
                return IfHandler.checking_mark(node.nextSibling)
            else:
                if node.nextSibling.nodeName == 'else':
                    return True
                else:
                    return False
        return False



from aestate.util import others, sqlOpera
from aestate.util.Log import ALog

alog = ALog
Alog = ALog
timeUtil = others
timeutil = others
cov = others
pack = others

sqlText = sqlOpera.TextUtil
sql_text = sqlOpera.TextUtil
sqltext = sqlOpera.TextUtil
textUtil = sqlOpera.TextUtil
textutil = sqlOpera.TextUtil
text_util = sqlOpera.TextUtil



# -*- utf-8 -*-
from aestate.work.xmlhandler.base import AestateNode, parse_attributes
from aestate.work.xmlhandler.final import XML_KEY, XML_IGNORE_NODES


def parse_children(root, elements, params):
    """
    获取所有子节点
    """
    child_list = {}
    for i in elements:
        node_name = i.nodeName if hasattr(i, 'nodeName') else i.node.nodeName
        if node_name not in XML_IGNORE_NODES:
            if node_name not in child_list.keys():
                child_list[node_name] = []
            child_list[node_name].append(AestateXml(root=root, node=i, params=params))
    return child_list


class AestateXml:
    """
    xml对象
    """

    def __init__(self, root, node, params: None):
        self.root = root
        self.node = node
        self.params = params if params else {}
        self.children = parse_children(node, node.childNodes if hasattr(node, 'childNodes') else node.children, params)
        self.tags = {}
        self.attrs = parse_attributes(node.attributes if hasattr(node, 'attributes') else node.attrs)
        self.resultType = None

    def text(self, target_obj):

        texts = AestateNode(self.root, self.node)
        for root_index, root_value in enumerate(self.node.childNodes):
            t = AestateNode(self.root, root_value)
            if root_value.nodeName in XML_KEY.keys():
                obj = XML_KEY[root_value.nodeName](target_obj, self.params, AestateXml, self.root, root_value, XML_KEY,
                                                   XML_IGNORE_NODES)
                texts = obj.apply(texts=texts)
            elif root_value.nodeName in XML_IGNORE_NODES:
                texts.add(node=root_value, index=root_index)
            texts.extend(t)
        return texts

    @staticmethod
    def read_file(filename):
        """
        读取xml文件
        """
        from xml.dom import minidom
        build = minidom.parse(filename)
        root = build.documentElement
        return AestateXml(root=root, node=root, params=None)



import ctypes
import threading

from aestate.work.Modes import Singleton

dll_h = ctypes.windll.kernel32


class AestateLanguage():
    LANG = dll_h.GetSystemDefaultUILanguage()


class I18n:
    """
    国际化语言,在统一配置下的全局语言解决方案
    """

    def __init__(self, langs=None):
        if langs is None:
            langs = {}
        self.langs = {}
        self.langs.update(langs)

    _instance_lock = threading.RLock()

    def whileGet(self, next_names: list):
        """
        重复获取i18n的字典语句
        :return:
        """
        pass

    def t(self, name: str):
        if AestateLanguage.LANG not in self.langs.keys():
            AestateLanguage.LANG = 0x409
        if name not in self.langs.get(AestateLanguage.LANG).keys():
            raise Exception(f'i18n field is not exist:{name}')
            # return name
        return self.langs.get(AestateLanguage.LANG).get(name)

    def __new__(cls, *args, **kwargs):
        """
        单例管理缓存内容
        """
        instance = Singleton.createObject(cls)
        return instance


class ExceptionI18n(I18n):
    """

    """

    def __init__(self):
        super(ExceptionI18n, self).__init__(langs={
            # 中文(简体,中国)
            0x804: {
                '': '未知错误',
                'if_tag_not_test': 'if 标记中的属性`test` 缺少必需的结构',
                'xml_syntax_error': 'xml语法错误,不相等的逻辑运算符数量,在:%s',
                'before_else_not_if': '在 else 标签前面找不到 if 标签',
                'not_field_name': '被调用的方法中不存在名为 `%s` 的参数',
                'not_from_node_name': '无法从节点中找到名为 `%s` 的模板',
                'not_result_map': "找不到 resultMap 模板",
                "not_type": "无法从节点中找到名为 `type` 的属性",
                "module_not_found": "模块未找到",
            },
            # 英语
            0x409: {
                '': 'Unknow error',
                'if_tag_not_test': 'The attribute`test` in the if tag is missing a required structure',
                'xml_syntax_error': 'Xml syntax error, unequal number of logical operators, from:%s',
                'before_else_not_if': 'Cannot find the if tag in front of the else tag',
                'not_field_name': 'The parameter named `%s` does not exist in the called method',
                'not_from_node_name': 'The template named `%s` could not be found from the node',
                'not_result_map': "Can't find resultMap template",
                "not_type": "The attribute named `type` could not be found from the node",
                "module_not_found": "`%s` Module Not Found",
            },
        })

    @staticmethod
    def tt(name):
        return ExceptionI18n().t(name)


class InfoI18n(I18n):
    """
    提示语句国际化
    """

    def __init__(self):
        super(InfoI18n, self).__init__(langs={
            # 中文(简体,中国)
            0x804: {
                "statement": "执行语句",
                "parameters": "参数",
                "cacheResult": "缓存结果",
                "selectResult": "查询结果",
                "updateResult": "受影响行",
            },
            0x409: {
                "statement": "Statement",
                "parameters": "Parameters",
                "cacheResult": "Cache Result",
                "selectResult": "Select Result",
                "updateResult": "Update Row",
            }
        })

    @staticmethod
    def tt(name):
        return InfoI18n().t(name)



# -*- utf-8 -*-
import re


class TextUtil(object):
    """
    sql操作工具
    """

    @staticmethod
    def replace_antlr(sql, **kwargs):
        """
        :param sql:具有特殊含义的sql语句
        """
        # #{}使用%s隔离
        sub_sql = re.sub(r'#{(.*?)}', '%s', sql)
        context_hashtag = re.findall(r'#{(.*?)}', sql)
        new_args = [kwargs[i] for i in context_hashtag]

        # ${}直接替换
        context_dollar = re.findall(r'\${(.*?)}', sub_sql)
        for cd in context_dollar:
            # 只有int需要转换
            if isinstance(kwargs[cd], int):
                kwargs[cd] = str(kwargs[cd])
            sub_sql = sub_sql.replace('${' + cd + '}', kwargs[cd])
        return sub_sql, new_args



import traceback
from enum import Enum


class DBException(Exception):
    def __init__(self, baseException):
        self.baseException = baseException
        self.baseName = self.baseException.__class__.__name__
        self.args = baseException.args
        super(DBException, self).__init__()

    def __str__(self):
        return f"({self.baseName}):{str(self.baseException)}"

    def println(self):
        traceback.print_exc(limit=1)


class ModuleCreateError(ModuleNotFoundError):
    """模块创建异常"""


class FieldNotExist(AttributeError):
    """字段不存在"""


class SqlSyntaxError(Exception):
    """sql语法"""


class SqlResultError(DBException):
    """sql返回错误"""


class XmlParseError(Exception):
    """xml解析错误"""


class NotFindTemplateError(XmlParseError):
    """找不到xml模板"""


class TagAttributeError(XmlParseError):
    """节点属性错误"""


class TagHandlerError(XmlParseError):
    """节点处理方式错误"""


class BaseSqlError:
    def __init__(self, exception):
        self.exception = exception
        self.text = str(exception)

    def raise_exception(self, **kwargs):
        self.exception.args = list(self.exception.args)
        # DBException(self.exception).println()
        raise DBException(self.exception)


class LogStatus(Enum):
    Error = 'ERROR'
    Warn = 'WARN'
    Info = 'INFO'



import datetime
import os
import sys
import threading
import traceback

from aestate.util.others import write, logTupleToText
from aestate.work.Cache import LogCache
from aestate.work.Modes import Singleton
from aestate.exception import LogStatus
from aestate.util import others
from aestate.work.commands import __log_logo__


class FieldsLength:
    DATETIME_FORMAT = 27
    INFO_FORMAT = 5
    LINE_FORMAT = 5
    OPERATION_FORMAT = 14
    # HEX_FORMAT = 17
    TASK_FORMAT = 7
    # CLASS_FORMAT = 70
    MSG_FORMAT = 0


class ConsoleColor:
    """
    控制台类型
    """

    class FontColor:
        # 黑色
        BLACK = 30
        # 灰色
        GRAY = 90
        # 粉色
        PINK = 31
        # 红色
        RED = 35
        # 绿色
        GREEN = 32
        # 浅绿色
        LIGHT_GREEN = 91
        # 黄色
        YELLOW = 33
        # 浅黄色
        LIGHT_YELLOW = 92
        # 深黄色
        DARK_YELLOW = 93
        # 紫色
        PURPLE = 34
        # 浅紫色
        LIGHT_PURPLE = 96
        # 青蓝色
        CYAN = 36
        # 白色
        WHITE = 37
        # 成功的颜色 和 info的颜色
        SUCCESS_COLOR = GREEN
        # 失败的颜色 和 错误的颜色
        ERROR_COLOR = RED
        # 警告的颜色
        WARNING_COLOR = YELLOW

    class BackgroundColor:
        # 黑色
        BLACK = 40
        # 红色
        RED = 41
        # 绿色
        GREEN = 42
        # 黄色
        YELLOW = 43
        # 蓝色
        BLUE = 44
        # 紫红色
        FUCHSIA = 45
        # 青蓝色
        CYAN = 46
        # 白色
        WHITE = 47

    class ShowType:
        # 默认
        DEFAULT = 0
        # 高亮
        HIGHLIGHT = 1
        # 下划线
        UNDERSCORE = 4
        # 闪烁
        FLASHING = 5
        # 反显
        REVERSE = 7
        # 不可见
        INVISIBLE = 8


class ConsoleWrite:
    def __init__(self):
        self.fontColor = ConsoleColor.FontColor.GREEN
        self.showType = ConsoleColor.ShowType.DEFAULT
        self.backColor = None

    # @staticmethod
    # def write(messages, consoleWriteObj=None):
    #     prefix = "{};".format(consoleWriteObj.showType) if consoleWriteObj.showType is not None else ""
    #     center = ";".format(consoleWriteObj.backColor) if consoleWriteObj.backColor is not None else ""
    #     suffix = "{}m{}".format(consoleWriteObj.fontColor, messages)
    #     out = "\033[{}{}{}\033[0m".format(prefix, center, suffix)
    #     print(out)

    @staticmethod
    def format_color(text, color=None):
        if color is not None:
            prefix = "{};".format(ConsoleColor.ShowType.DEFAULT)
            suffix = "{}m{}".format(color, text)
            out = "\033[{};{}\033[0m".format(prefix, suffix)
            return out
        return text


class ALog(object):
    _instance_lock = threading.RLock()

    def __init__(self, path, print_flag=False, save_flag=False, max_clear=10):
        """

        初始化配置

        :param path:保存的路径

        :param print_flag:是否打印日志 默认False

        :param save_flag:是否保存日志 默认False

        :param max_clear:日志储存最大限制,默认10MB 单位:MB

        """
        self.max_clear = max_clear * 1024 * 1000
        self.path = path
        self.print_flag = print_flag
        self.save_flag = save_flag

    @staticmethod
    def pure_log(msg, **kwargs):
        """
        输出任务执行日志

        :param msg:消息

        """
        ALog.log(msg=msg, **kwargs)

    @staticmethod
    def format_text(field: LogStatus, line, obj, task_name, msg, ned_text=False,
                    text_color: ConsoleColor.FontColor = None):
        """
        将字符串格式化成好看的颜色
        """
        try:
            if obj is not None:
                write_repr = others.fullname(obj)
            else:
                write_repr = 'OBJECT IS NULL'
        except TypeError:
            write_repr = 'OBJECT CAN`T NOT PARSE'
        t = datetime.datetime.utcnow().strftime('%Y-%m-%d %H:%M:%S.%f')
        pure_text = ' '.join([str(t), str(field.value), str(line), str(hex(id(obj))),
                              '[{}]'.format(task_name), str(write_repr), f" : {msg}"])
        t = ConsoleWrite.format_color(f"{t}".ljust(FieldsLength.DATETIME_FORMAT), ConsoleColor.FontColor.CYAN)
        _field = ConsoleWrite.format_color(f"{field.value}".rjust(FieldsLength.INFO_FORMAT),
                                           ConsoleColor.FontColor.GREEN
                                           if field == LogStatus.Info
                                           else ConsoleColor.FontColor.RED
                                           if field == LogStatus.Error
                                           else ConsoleColor.FontColor.YELLOW
                                           if field == LogStatus.Warn
                                           else ConsoleColor.FontColor.YELLOW)
        line = f"{line}".rjust(FieldsLength.LINE_FORMAT)
        hex_id = ConsoleWrite.format_color(f" {str(hex(id(obj)))}", ConsoleColor.FontColor.PINK)
        task_name = ConsoleWrite.format_color(f"{task_name}".rjust(FieldsLength.TASK_FORMAT),
                                              ConsoleColor.FontColor.PURPLE)
        write_repr = ConsoleWrite.format_color(write_repr,
                                               ConsoleColor.FontColor.LIGHT_GREEN
                                               if field != LogStatus.Error
                                               else ConsoleColor.FontColor.RED)
        msg = ConsoleWrite.format_color(f" : {msg}", text_color)
        info = "{}{}{}{}{}{}{}".format(t, _field, line, hex_id, ' [{}] '.format(task_name), write_repr, msg)
        if ned_text:
            return info, pure_text
        return info

    @staticmethod
    def log(msg, obj=None, line=sys._getframe().f_back.f_lineno, task_name='TEXT', LogObject=None,
            field: LogStatus = LogStatus.Info, func=None,
            text_color: ConsoleColor.FontColor = None, **kwargs):
        """
        输出任务执行日志

        :param msg:消息
        :param obj:执行日志的对象地址
        :param line:被调用前的行数
        :param task_name:任务对象的值
        :param LogObject:写出文件的对象
        :param field:日志模式
        :param func:日志执行后的自定义操作
        :param text_color: 文本颜色
        """

        try:
            if obj is not None:
                write_repr = others.fullname(obj)
            else:
                write_repr = 'OBJECT IS NULL'
        except TypeError:
            write_repr = 'OBJECT CAN`T NOT PARSE'

        # write_repr = repr if repr and not repr_c else repr_c[0] if repr_c else type(obj)
        # 格式：时间 类型 被调用时行数 对象地址 日志信息 执行类 信息
        t = datetime.datetime.utcnow().strftime('%Y-%m-%d %H:%M:%S.%f')[:-3]
        con_text = ' '.join([str(t), str(field.value), str(line), str(hex(id(obj))),
                             '[{}]'.format(task_name), str(write_repr), f" : {msg}"])
        info = ALog.format_text(field, line, obj, task_name, msg, text_color=text_color)

        # print(info)

        def __log_obj_write__(_object):
            if _object is not None:
                if field == LogStatus.Info:
                    _object.info(con_text, pure_text=info, line=line, obj=obj)
                elif field == LogStatus.Error:
                    _object.error(con_text, pure_text=info, line=line, obj=obj)
                elif field == LogStatus.Warn:
                    _object.warn(con_text, pure_text=info, line=line, obj=obj)
                else:
                    _object.info(con_text, pure_text=info, line=line, obj=obj)
            else:
                print(info)

        if obj is not None:
            if hasattr(obj, 'log_obj'):
                __log_obj_write__(obj.log_obj)
            else:
                __log_obj_write__(LogObject)
        else:
            __log_obj_write__(LogObject)
        if func is not None:
            func(con_text)

        return info

    @staticmethod
    def warning(**kwargs):
        ALog.log(task_name='WARNING', field=LogStatus.Warn, **kwargs)

    @staticmethod
    def log_error(msg=None, obj=None, line=sys._getframe().f_back.f_lineno, task_name='ERROR',
                  LogObject=None, raise_exception=False):
        """
        :param msg:描述
        :param line:行
        :param obj:执行的对象，当允许抛出异常时，则指明该对象为一个Exception或他的子类
        :param task_name:线程唯一名称
        :param LogObject:日志对象
        :param raise_exception:是否抛出异常
        """
        text = list(msg)

        def get_stack():
            text.append('\n')
            exc_type, exc_value, exc_traceback_obj = sys.exc_info()
            extracted_list = traceback.extract_tb(exc_traceback_obj)
            for item in traceback.StackSummary.from_list(extracted_list).format():
                text.append(item)

        if raise_exception:
            if isinstance(obj, type):
                try:
                    raise obj(msg)
                except obj:
                    get_stack()
                    text.append(f'{obj.__name__} :{msg}')
            else:
                get_stack()
                text.append(f'{obj.__class__.__name__} :{msg}')
        ALog.log(msg=''.join(text), obj=obj, line=line, task_name=task_name,
                 LogObject=LogObject if LogObject is not None else None, field=LogStatus.Error,
                 text_color=ConsoleColor.FontColor.RED)

    def template(self, status: LogStatus, *content, **kwargs):
        """
        日志的模板,error、info和warn都在这里执行
        """
        # 从缓存中获取日志的对象
        log_cache = LogCache()
        # 从缓存中获取文件名,因为不能每次查询都生成一个文件,这样做是不合理的
        _path = log_cache.get_filename(self.path, self.max_clear, status)
        if status == LogStatus.Info:
            logo_show = 'info_logo_show'
        elif status == LogStatus.Warn:
            logo_show = 'warn_logo_show'
        elif status == LogStatus.Error:
            logo_show = 'error_logo_show'
        else:
            logo_show = 'info_logo_show'
        # 如果日志中没有打印过logo,就写入logo
        ls = getattr(log_cache, logo_show)
        if not ls:
            setattr(log_cache, logo_show, True)
            self.log_util(_path, __log_logo__)
        self.log_util(_path, *content)
        line = kwargs['line'] if 'line' in kwargs.keys() else sys._getframe().f_back.f_lineno
        obj = kwargs['obj'] if 'obj' in kwargs.keys() else self
        text = kwargs['pure_text'] \
            if 'pure_text' in kwargs.keys() \
            else ALog.format_text(status, line, obj, status.value, logTupleToText(False, *content))
        print(text)

    def info(self, *content, **kwargs):
        """
        成功日志
        :param content:内容
        :return:
        """
        self.template(LogStatus.Info, *content, **kwargs)

    def warn(self, *content, **kwargs):
        """
        警告日志
        :param content:内容
        :return:
        """
        self.template(LogStatus.Warn, *content, **kwargs)

    def error(self, *content, **kwargs):
        """
        错误日志
        :param content:内容
        :return:
        """
        self.template(LogStatus.Error, *content, **kwargs)

    def log_util(self, path_str, *content):
        """
        日志工具
        :param path_str:
        :param content:
        :return:
        """
        path = self.get_path(path_str)
        if self.save_flag:
            write(path, *content)

    def get_path(self, end_path):
        """
        日志类获取绝对路径
        :param end_path:
        :return:
        """
        _STATIC_TXT = os.path.join('', self.path + end_path)
        return _STATIC_TXT

    def __new__(cls, *args, **kwargs):
        instance = Singleton.createObject(cls)
        return instance


class logging(object):

    @classmethod
    def gen(cls, _object) -> ALog:
        return _object.log_obj



# -*- utf-8 -*-
# encoding:utf-8
# @Time: 2021/6/27 20:47
# @Author: CACode
# 版本有三种状态 正式版从1.0.0往后逐个加 1,对应版本的补丁为'a+补丁次数'
__version__ = '1.0.8'
__description__ = "Aestate framework for Python,You can see:https://gitee.com/aecode/aestate"
__author__ = "CACode"
__author_email__ = "cacode@163.com"
__url__ = "https://gitee.com/aecode/aestate"
__issues__ = 'https://gitee.com/aecode/aestate/issues'
__license__ = 'Apache License 2.0'
__project_name__ = 'Aestate'
__logo__ = """
         :: Aestate Framework ::      (version:%s)
    +    __                    _        _        __    +
    +   / /     /\            | |      | |       \ \   +
    +  / /     /  \   ___  ___| |_ __ _| |_ ___   \ \  +
    + | |     / /\ \ / _ \/ __| __/ _` | __/ _ \   | | +
    +  \ \   / ____ \  __/\__ \ || (_| | ||  __/  / /  +
========\_\=/_/====\_\___||___/\__\__,_|\__\___|=/_/========
""" % __version__

__log_logo__ = """
         :: Aestate Framework ::      (version:%s)
       __      ____    ___    ____      __      ____    ____ 
      /__\    ( ___)  / __)  (_  _)    /__\    (_  _)  ( ___)
     /(__)\    )__)   \__ \    )(     /(__)\     )(     )__) 
    (__)(__)  (____)  (___/   (__)   (__)(__)   (__)   (____)
""" % __version__

import importlib
import os.path

try:
    from prettytable import PrettyTable
except ModuleNotFoundError as e:
    print("请先安装 [prettytable] 再执行 [-h] 命令,使用 [pip install prettytable]")


class Commands:
    def __init__(self, *args):
        """
        下面的@staticmethod主要是为了不想看见黄线警告，并没有其他意思
        """
        self.args = args
        self.c = {
            "": (
                self.start,
                '显示aestate的logo和版本号，用于检查aestate是否安装成功',
                "aestate"
            ),
            "-v": (
                self.version,
                "显示aestate的版本号",
                "aestate -v"
            ),
            # "-create": (
            #     self.create,
            #     "将文件内存在pojo对象的类生成到数据库中称为数据库的表"
            #     "数据库格式化类型参考默认的 [mysql] 格式",
            #     'aestate -create [文件名] [数据库类型 (可选)]'
            # ),
            # "-m": (
            #     self.make,
            #     "将数据库中的表同步生成到当前目录下的 [model.py]，并默认命名为 [数据库命_表名]",
            #     'aestate -m [--n [生成的文件名 (可选) ]] [--nn [生成的类名 (可选)]]'
            # ),
            # "-enc": (
            #     self.enc,
            #     "加密模型",
            #     'aestate -enc [密码]'
            # ),
            # "-dec": (
            #     self.dec,
            #     "解密模型",
            #     'aestate -dec [被加密后的文件] [密码]'
            # ),
            # "-check": (
            #     self.check,
            #     "检查模型与数据库中的表结构是否一直",
            #     'aestate -check [文件名] [数据库名]'
            # ),
            # "-h": (
            #     self.help,
            #     "帮助文档",
            #     'aestate -h'
            # ),
            # "-g": (
            #     self.generate,
            #     "在当前目录下生成一个",
            #     'aestate -gc 项目名'
            # )
        }

    def generate(self):
        code = '''
from aestate.work.command import Generate
from 
models:List[Pojo] = []
if __name__ == '__main__':
    Generate().start(models)
        '''
        if not os.path.exists('start.py') or not os.path.isfile('start.py'):
            with open('start.py', 'w') as f:
                f.write(code)

    def start(self):
        print(__logo__)

    def create(self):
        print(__logo__)
        try:
            file = self.args[2]
            db_name = self.args[3]
        except IndexError:
            raise IndexError("为了保证数据库的sql执行顺利，请填写pojo存在的文件名和数据库名称")
        import inspect
        temp_module = importlib.import_module(file)

        temp_classes = inspect.getmembers(temp_module, inspect.isclass)
        for name, class_ in temp_classes:
            c = class_()
            c.orm.create()

    def enc(self):
        pass

    def dec(self):
        pass

    def version(self):
        print(__version__)

    def make(self):
        pass

    def check(self):
        print(__logo__)
        try:
            file = self.args[2]
            db_name = self.args[3]
        except IndexError:
            raise IndexError("为了保证数据库的sql执行顺利，请填写pojo存在的文件名和数据库名称")
        import inspect
        temp_module = importlib.import_module(file)

        temp_classes = inspect.getmembers(temp_module, inspect.isclass)
        for name, class_ in temp_classes:
            c = class_()
            c.orm.check()

    def help(self):
        table = PrettyTable(["命令", "使用方法", "描述"])
        table.border = True
        table.junction_char = '-'
        [table.add_row([k, v[2], v[1]]) for k, v in self.c.items()]
        print(table)


class Generate:
    def start(self):
        pass


if __name__ == '__main__':
    Generate().start()



# -*- utf-8 -*-
from aestate.work.xmlhandler.nodes import SelectNode, \
    UpdateNode, IfNode, ElseNode, SwitchNode, IncludeNode

XML_IGNORE_NODES = ['#text']
XML_KEY = {
    'select': SelectNode,
    'update': UpdateNode,
    'if': IfNode,
    'else': ElseNode,
    'switch': SwitchNode,
    'include': IncludeNode,
}



# -*- utf-8 -*-
from _ast import AST


class Attribute:
    def __init__(self, name, node):
        self.root = node
        self.text = node.value
        self.name = name


class TextNode:
    def __init__(self, root, node, index):
        self.root = root
        self.node = node
        self.index = index
        self.childNodes = node.childNodes if hasattr(node, 'childNodes') else None
        self.attributes = node.attributes if hasattr(node, 'attributes') else None

    @property
    def text(self):
        return self.node.text if hasattr(self.node, 'text') else self.node.data

    def __str__(self):
        return self.node.text


class AestateNode(list):
    def __init__(self, root, node):
        list.__init__([])
        self.root = root
        self.node = node
        # 给定一个标记，让剩下的标签允许存放信息
        self.mark = {}

    def add(self, node, index) -> None:
        self.append(TextNode(self.root, node, index))

    @property
    def text(self):
        texts = [i.text for i in self]
        # for i, v in enumerate(arr):
        #     if v.node.nodeName in XML_KEY.keys():
        #         obj = XML_KEY[v.node.nodeName](v)
        #         texts.append(obj.pure_str(XML_KEY))
        #     elif v.node.nodeName in XML_TEXT_NODE:
        #         texts.append(v.node.data)
        return ' '.join(texts)

    def __str__(self) -> str:
        return self.text


def parse_attributes(attr):
    if attr is None:
        return {}
    objs = {}
    for k, v in attr._attrs.items():
        objs[k] = Attribute(name=k, node=v)
    return objs


class ResultAST(AST):
    """返回值的ast抽象语法树"""

    @staticmethod
    def parse(resultMap):
        pass



# -*- utf-8 -*-
import functools
import math
import os
import threading
import time
from collections import OrderedDict
from enum import Enum

import psutil

from aestate.exception import LogStatus
from aestate.util import others
from aestate.util.others import write, save_rb_tree
from aestate.work.Modes import Singleton

"""
全世界无产者,联合起来!!!
我们要强烈抵制资本主义的剥削!!!
"""


def tree_log(func):
    @functools.wraps(func)
    def function(a, b):
        save_rb_tree(a.root, "{}-{}".format(a.index, a.action))
        a.index += 1
        func(a, b)
        save_rb_tree(a.root, "{}-{}".format(a.index, a.action))
        a.index += 1

    return function


class CacheStatus(Enum):
    """
    缓存的状态
    """
    CLOSE = 0
    OPEN = 1


class SqlCacheItem(object):
    """缓存对象"""

    def __init__(self, key, value, instance):
        # 执行的sql
        self.sql = key
        # 长度,判断左右子树
        self.length = len(key)
        # 数据
        self.data = value
        self.instance = instance
        # 使用次数
        self.using_count = 0
        # 写入的时间
        self.create_time = time.time()
        self.last_using_time = time.time()
        # 左子树
        self.lchild = None
        # 右子树
        self.rchild = None
        # 删除标记, True为被删除,不会被搜索到
        self.is_delete = False
        # 平衡差,为左子树层级减去右子树层级，介于[-1,1]为合法值
        self.bf = 0
        super(SqlCacheItem, self).__init__()

    def __setattr__(self, key, value):
        if key == 'using_count':
            self.last_using_time = time.time()
        super(SqlCacheItem, self).__setattr__(key, value)

    def set_using_count(self):
        self.using_count += 1

    def get_sql(self):
        return self.sql

    def get_value(self):
        return self.data

    def set_value(self, value):
        self.data = value

    def is_black_node(self):
        return self.color == "B"

    def is_red_node(self):
        return self.color == "R"

    def set_black_node(self):
        self.color = "B"

    def set_red_node(self):
        self.color = "R"

    def print(self):
        if self.lchild:
            self.lchild.print()
        print(id(self))
        if self.rchild:
            self.rchild.print()


class DataContainer:
    """红黑树 数据容器"""

    def left_rotate(self, node):
        """
         * 左旋示意图：对节点x进行左旋
         *     parent               parent
         *    /                       /
         *   node                   right
         *  / \                     / \
         * ln  right   ----->     node  ry
         *    / \                 / \
         *   ly ry               ln ly
         * 左旋做了三件事：
         * 1. 将right的左子节点ly赋给node的右子节点,并将node赋给right左子节点ly的父节点(ly非空时)
         * 2. 将right的左子节点设为node，将node的父节点设为right
         * 3. 将node的父节点parent(非空时)赋给right的父节点，同时更新parent的子节点为right(左或右)
        :param node: 要左旋的节点
        :return:
        """
        parent = node.parent
        right = node.right
        # 把右子子点的左子点节   赋给右节点 步骤1
        node.right = right.left
        if node.right:
            node.right.parent = node
        # 把 node 变成基右子节点的左子节点 步骤2
        right.left = node
        node.parent = right
        # 右子节点的你节点更并行为原来节点的父节点。 步骤3
        right.parent = parent
        if not parent:
            self.root = right
        else:
            if parent.left == node:
                parent.left = right
            else:
                parent.right = right
        pass

    def right_rotate(self, node):
        """
         * 右旋示意图：对节点y进行右旋
         *        parent           parent
         *       /                   /
         *      node                left
         *     /    \               / \
         *    left  ry   ----->   ln  node
         *   / \                     / \
         * ln  rn                   rn ry
         * 右旋做了三件事：
         * 1. 将left的右子节点rn赋给node的左子节点,并将node赋给rn右子节点的父节点(left右子节点非空时)
         * 2. 将left的右子节点设为node，将node的父节点设为left
         * 3. 将node的父节点parent(非空时)赋给left的父节点，同时更新parent的子节点为left(左或右)
        :param node:
        :return:
        """
        parent = node.parent
        left = node.left

        # 处理步骤1
        node.left = left.right
        if node.left:
            node.left.parent = node

        # 处理步骤2
        left.right = node
        node.parent = left

        # 处理步骤3
        left.parent = parent
        if not parent:
            self.root = left
        else:
            if parent.left == node:
                parent.left = left
            else:
                parent.right = left
        pass

    def insert_node(self, node):
        """
        二叉树添加往红黑树中添加一个红色节点
        :param node:
        :return:
        """
        if not self.root:
            self.root = node
            return

        cur = self.root
        while cur:
            if cur.val < node.val:
                if not cur.right:
                    node.parent = cur
                    cur.right = node
                    break
                cur = cur.right
                continue

            if cur.val > node.val:
                if not cur.left:
                    node.parent = cur
                    cur.left = node
                    break
                cur = cur.left
        pass

    @tree_log
    def check_node(self, node):
        """
        检查节点及父节是否破坏了
        性质二：根节点是黑色；
        性质四：每个红色节点的两个子节点都是黑色的（也就是说不存在两个连续的红色节点）；
        @@ 性质四可反向理解为， 节点和其父点必定不能够同时为红色节点
        :param node:
        :return:
        """
        # 如果是父节点直接设置成黑色节点，退出
        if self.root == node or self.root == node.parent:
            self.root.set_black_node()
            print("set black ", node.val)
            return

        # 如果父节点是黑色节点，直接退出
        if node.parent.is_black_node():
            return

        # 如果父节点的兄弟节点也是红色节点,
        grand = node.parent.parent
        if not grand:
            self.check_node(node.parent)
            return
        if grand.left and grand.left.is_red_node() and grand.right and grand.right.is_red_node():
            grand.left.set_black_node()
            grand.right.set_black_node()
            grand.set_red_node()
            self.check_node(grand)
            return

        # 如果父节点的兄弟节点也是黑色节点,
        # node node.parent node.parent.parent 不同边
        parent = node.parent
        if parent.left == node and grand.right == node.parent:
            self.right_rotate(node.parent)
            self.check_node(parent)
            return
        if parent.right == node and grand.left == node.parent:
            parent = node.parent
            self.left_rotate(node.parent)
            self.check_node(parent)
            return

        # node node.parent node.parent.parent 同边
        parent.set_black_node()
        grand.set_red_node()
        if parent.left == node and grand.left == node.parent:
            self.right_rotate(grand)
            return
        if parent.right == node and grand.right == node.parent:
            self.left_rotate(grand)
            return

    def add_node(self, node):
        self.action = 'inser node {}'.format(node.val)
        self.insert_node(node)
        self.check_node(node)
        pass

    def check_delete_node(self, node):
        """
        检查删除节点node
        :param node:
        :return:
        """
        if self.root == node or node.is_red_node():
            return

        node_is_left = node.parent.left == node
        brother = node.parent.right if node_is_left else node.parent.left
        # brother 必不为空
        if brother.is_red_node():
            # 如果是黑色节点，兄弟节点是红色节点， 旋转父节点： 把你节点变成黑色，兄弟节点变黑色。 重新平衡
            if node_is_left:
                self.left_rotate(node.parent)
            else:
                self.right_rotate(node.parent)
            node.parent.set_red_node()
            brother.set_black_node()
            print("check node delete more ")
            # 再重新检查当前节点
            self.check_delete_node(node)
            return

        all_none = not brother.left and not brother.right
        all_black = brother.left and brother.right and brother.left.is_black_node() and brother.right.is_black_node()
        if all_none or all_black:
            brother.set_red_node()
            if node.parent.is_red_node():
                node.parent.set_black_node()
                return
            self.check_delete_node(node.parent)
            return

        # 检查兄弟节点的同则子节点存丰并且是是红色节点
        brother_same_right_red = node_is_left and brother.right and brother.right.is_red_node()
        brother_same_left_red = not node_is_left and brother.left and brother.left.is_red_node()
        if brother_same_right_red or brother_same_left_red:

            if node.parent.is_red_node():
                brother.set_red_node()
            else:
                brother.set_black_node()
            node.parent.set_black_node()

            if brother_same_right_red:
                brother.right.set_black_node()
                self.left_rotate(node.parent)
            else:
                brother.left.set_black_node()
                self.right_rotate(node.parent)

            return

        # 检查兄弟节点的异则子节点存丰并且是是红色节点
        brother_diff_right_red = not node_is_left and brother.right and brother.right.is_red_node()
        brother_diff_left_red = node_is_left and brother.left and brother.left.is_red_node()
        if brother_diff_right_red or brother_diff_left_red:
            brother.set_red_node()
            if brother_diff_right_red:
                brother.right.set_black_node()
                self.left_rotate(brother)
            else:
                brother.left.set_black_node()
                self.right_rotate(brother)

            self.check_delete_node(node)
            return

    def pre_delete_node(self, node):
        """
        删除前检查，返回最终要删除的点
        :param node:
        :return:
        """
        post_node = self.get_post_node(node)
        if post_node:
            node.val, post_node.val = post_node.val, node.val
            return self.pre_delete_node(post_node)
        pre_node = self.get_pre_node(node)
        if pre_node:
            pre_node.val, node.val = node.val, pre_node.val
            return self.pre_delete_node(pre_node)
        # 没有前驱节点，也没有后续节点
        return node

    def get_pre_node(self, node):
        """
        获取 前驱 节点 ， 树中比node小的节点中最大的值
        :param node:
        :return:
        """
        if not node.left:
            return None
        pre_node = node.left
        while pre_node.right:
            pre_node = pre_node.right
        return pre_node

    def get_post_node(self, node):
        """
        获取后续节点:
        :param node:树中比node大的节点中最小的值
        :return:
        """
        if not node.right:
            return None
        post_node = node.right
        while post_node.left:
            post_node = post_node.left
        return post_node

    def get_node(self, val):
        """
        根据值查询节点信息
        :param val:
        :return:
        """
        if not self.root:
            return None
        node = self.root
        while node:
            if node.val == val:
                break
            if node.val > val:
                node = node.left
                continue
            else:
                node = node.right
        return node

    def delete_node(self, val):

        node = self.get_node(val)
        if not node:
            print("node error {}".format(val))
            return
        save_rb_tree(self.root, "{}_delete_0".format(val))
        # 获取真正要删除的节点
        node = self.pre_delete_node(node)
        save_rb_tree(self.root, "{}_delete_1".format(val))
        # node 节点必不为空，且子节点也都为空
        self.check_delete_node(node)
        save_rb_tree(self.root, "{}_delete_2".format(val))
        # 真正删除要删除的节点
        self.real_delete_node(node)
        save_rb_tree(self.root, "{}_delete_3".format(val))
        pass

    def real_delete_node(self, node):
        """
        真正删除节点函数
        :param node:
        :return:
        """
        if self.root == node:
            self.root = None
            return
        if node.parent.left == node:
            node.parent.left = None
            return
        if node.parent.right == node:
            node.parent.right = None
        return

    # -----------------------

    @staticmethod
    def left_whirl(node):
        """
        左旋
        当node值为-2的时候可以执行此操作使其节点值为0,node为最小不平衡树的根节点
        :param node:
        :return:
        """
        node.bf = node.rchild.bf = 0

        node_right = node.rchild
        node.rchild = node.rchild.lchild
        node_right.lchild = node
        return node_right

    @staticmethod
    def right_whirl(node):
        """
        右旋
        当node值为2的时候可以执行此操作使其节点值为0,node为最小不平衡树的根节点
        :param node:
        :return:
        """
        node.bf = node.lchild.bf = 0
        node_left = node.lchild
        node.lchild = node.lchild.rchild
        node_left.rchild = node
        return node_left

    @staticmethod
    def left_right_whirl(node):
        """
        左右旋,先左旋子节点,再右旋node节点
        :param node:
        :return:
        """
        node_b = node.lchild
        node_c = node_b.rchild
        node.lchild = node_c.rchild
        node_b.rchild = node_c.lchild
        node_c.lchild = node_b

        node_c.rchild = node

        if node_c.bf == 0:
            node.bf = node_b.bf = 0
        elif node_c.bf == 1:
            node.bf = -1
            node_b.bf = 0
        else:
            node.bf = 0
            node_b.bf = 1

        node_c.bf = 0
        return node_c

    @staticmethod
    def right_left_whirl(node):
        """
        右左旋,先右旋子节点,再左旋node节点
        :param node:
        :return:
        """
        node_b = node.rchild
        node_c = node_b.lchild

        node_b.lchild = node_c.rchild
        node.rchild = node_c.lchild
        node_c.rchild = node_b

        node_c.lchild = node

        if node_c.bf == 0:
            node.bf = node_b.bf = 0
        elif node_c.bf == 1:
            node.bf = 0
            node_b.bf = -1
        else:
            node.bf = 1
            node_b.bf = 0

        node_c.bf = 0
        return node_c

    def __init__(self):
        self.root = None
        self.index = 1
        self.action = ""

    def _search(self, key: str) -> SqlCacheItem or None:
        temp = self.root
        while temp:
            if temp.length == len(key) and temp.get_sql() == key:
                if temp.is_delete:
                    temp = None
                break
            elif temp.length > len(key):
                temp = temp.lchild
            else:
                temp = temp.rchild
        else:
            return None

        return temp

    def search(self, key):
        res = self._search(key)
        if res:
            return res
        return None

    def insert(self, targetNode: SqlCacheItem):
        key, value = targetNode.length, targetNode.data
        if not self.root:
            self.root = targetNode
            return
        mut_node, point = self.root, self.root
        mut_parent, p_parent = None, None
        while point:
            if point.length == key and point.get_sql() == targetNode.get_sql():
                point.set_value(value)
                return
            if point.bf != 0:
                mut_parent, mut_node = p_parent, point
            p_parent = point
            if key > point.length:
                point = point.rchild
            else:
                point = point.lchild
        if key > p_parent.length:
            p_parent.rchild = targetNode
        else:
            p_parent.lchild = targetNode
        ta = mut_node
        while ta:
            if ta.length == key:
                break
            elif key > ta.length:
                ta.bf -= 1
                ta = ta.rchild
            else:
                ta.bf += 1
                ta = ta.lchild
        if mut_node.length > key:
            if mut_node.lchild:
                p_pos = mut_node.lchild.length > key
            else:
                mut_node.lchild = targetNode
                p_pos = False
        else:
            if mut_node.rchild:
                p_pos = mut_node.rchild.length > key
            else:
                mut_node.rchild = targetNode
                p_pos = False
        if mut_node.bf > 1:
            if p_pos:
                mut_node = DataContainer.right_whirl(mut_node)
            else:
                mut_node = DataContainer.left_right_whirl(mut_node)
        elif mut_node.bf < -1:
            if p_pos:
                mut_node = DataContainer.right_left_whirl(mut_node)
            else:
                mut_node = DataContainer.left_whirl(mut_node)
        if mut_parent:
            if mut_parent.length > key:
                mut_parent.lchild = mut_node
            else:
                mut_parent.rchild = mut_node
        else:
            self.root = mut_node

    def delete(self, key):
        """
        删除节点并返回该节点的值
        :return:
        """
        p = self._search(key)
        if not p:
            return False
        p.is_delete = True
        return p.get_value()


class SqlCacheManage(object):
    """
    缓存管理

    1.当内存满足系统运行内存的1/10时,满足最大限度数据内容,保证数据完整性的同时保留数据

    2.当单次查询数据大于阈值时,保留数据并不在扩大缓存空间,数据完整保留,但不再清理,直到处于第二缓存空间更多查询数据量再次大于阈值时清理

    3.当通过aestate改变数据时记录数据变更信息,并重新将数据写入缓存,移除旧缓存数据,这将意味着非通过aestate修改的数据不可被检测到

    4.扩容策略:当前内存>=当前容量1/2时,重新计算查询数据量

    5.流量计算方式:当前缓存大小 + (当前缓存大小 / 上次扩容时间至当前时间段内插入的新内容数量) ** 2

    6.移除方案:时间段内缓存查询次数最少内存最大优先,当 (A次数-B次数) * 10 <= (A占用内存-B占用内存),优先删除B
    """
    # 初始内存大小为1024Byte
    __capacity_max__ = 1024
    # 系统运行时计算得到的内存阈值
    __max__ = psutil.virtual_memory().free / 10
    _instance_lock = threading.RLock()
    # 容器
    data_container = DataContainer()
    # 缓存的状态
    status = CacheStatus.OPEN

    def __contains__(self, o: str) -> bool:
        """判断缓存中是否存在这个sql的查询记录"""
        return self.data_container.search(o)

    def get_size(self):
        """获取当前缓存的大小"""
        return self.get_container_size()

    def get_max(self):
        return self.__max__

    def need_calculate(self):
        """是否需要清理缓存"""
        return self.get_capacity_max() / 2 <= self.get_size()

    def calculate_ram(self) -> bool:
        """扩容"""
        self.reset_max_ram()
        if self.need_calculate():
            target_ram = int(self.get_capacity_max() * 2 + self.get_size())
            if target_ram < self.get_max():
                self.__capacity_max__ = target_ram
                return True
            else:
                # 直接等于最大内存,然后清理一下
                self.__capacity_max__ = self.get_max()
                # 当前允许的最大内存的20%(缓存的平均值),直到缓存满足
                size = math.ceil((self.get_capacity_max() * 0.2) / (self.get_size() / len(self.data_container)))
                if size != 0:
                    while size:
                        del self.data_container[0]
                        size -= 1
                return True
        else:
            return False

    def get(self, sql) -> SqlCacheItem:
        """获取一条sql"""
        target = self.index(sql)
        target.set_using_count()
        return target

    def remove(self, sql):
        """移除某个sql的缓存"""
        index = self.index(sql)
        if index != -1:
            del self.data_container[index]

    def remove_by_instance(self, tb_name):
        """根据instance的表来删除缓存"""
        self.data_container.delete(tb_name)

    def clean_up(self):
        """清理缓存,不是清除缓存,清理是清理使用次数不多的缓存"""
        if self.need_calculate():
            # 先试图扩容
            self.calculate_ram()

    def reset_max_ram(self):
        """重新计算当前可用的最大缓存"""
        free_ram = psutil.virtual_memory().free
        self.__max__ = int(free_ram / 10)

    def set(self, sql, value, instance):
        self.data_container.insert(targetNode=SqlCacheItem(key=sql, value=value, instance=instance))
        # 判断缓存是否已经满了
        self.clean_up()

    def get_container(self) -> DataContainer:
        return self.data_container

    def get_container_size(self):
        return self.data_container.__sizeof__()

    def get_capacity_max(self):
        """获取当前内存允许的最大限制"""
        return self.__capacity_max__

    def clear(self):
        """清空缓存,谨慎操作
        如果仅仅是需要清理缓存空间,请使用clean_up()函数
        """
        self.data_container.root = None

    def index(self, sql):
        """验证sql是否存在缓存"""
        return self.data_container.search(sql)

    def __new__(cls, *args, **kwargs):
        """
        单例管理缓存内容
        """
        instance = Singleton.createObject(cls)
        return instance


class PojoContainer:
    """对象管理器"""

    def __init__(self):
        self.solvent = []

    def __add__(self, __object):
        self.solvent.append(__object)

    @property
    def size(self) -> int:
        return len(self.solvent)

    def get(self, name):
        for item in self.solvent:
            if item._type == name:
                return item._object
        return None


class PojoItemCache(OrderedDict):
    """单个对象的容器"""

    def __init__(self, _type, _object):
        super(PojoItemCache).__init__()
        self._type = _type
        self._object = _object


class PojoManage:
    """管理pojo的缓存"""
    _instance_lock = threading.RLock()
    pojo_list = PojoContainer()

    def append(self, _cls_name: type, _object):
        self.pojo_list + PojoItemCache(_type=_cls_name, _object=_object)

    @staticmethod
    def get(_cls, *args, **kwargs):
        if 'new' in kwargs.keys() and kwargs['new'] == True:
            return object.__new__(_cls)
        this = PojoManage()
        _class_object_ = object.__new__(_cls)
        cls_name = others.fullname(_class_object_)
        _obj = this.pojo_list.get(cls_name)
        if _obj is None:
            this.append(cls_name, _class_object_)
            _obj = this.pojo_list.get(cls_name)
        [setattr(_obj, k, v) for k, v in kwargs.items()]
        return _obj

    def __new__(cls, *args, **kwargs):
        instance = Singleton.createObject(cls)
        return instance


class LogCache:
    _instance_lock = threading.RLock()
    # 是否已经显示logo了
    info_logo_show = False
    warn_logo_show = False
    error_logo_show = False
    # 文件名,当满足最大时将会使用一个新的文件作为储存日志
    info_file_name = []
    warn_file_name = []
    error_file_name = []

    def get_filename(self, path, max_clear, status):

        if status == LogStatus.Info:
            center_name = 'info'
            oa = self.info_file_name
            logo_show = 'info_logo_show'
        elif status == LogStatus.Error:
            center_name = 'error'
            oa = self.warn_file_name
            logo_show = 'error_logo_show'
        elif status == LogStatus.Warn:
            center_name = 'warn'
            oa = self.error_file_name
            logo_show = 'warn_logo_show'
        else:
            center_name = 'info'
            oa = self.info_file_name
            logo_show = 'info_logo_show'

        _path = os.path.join(path, center_name)
        if len(oa) == 0:
            oa.append(others.date_format(fmt='%Y.%m.%d.%H.%M.%S') + '.log')
            setattr(self, logo_show, False)
        else:
            if not os.path.exists(os.path.join(_path, oa[len(oa) - 1])):
                write(os.path.join(_path, '.temp'), '')
                setattr(self, logo_show, False)
            if os.path.getsize(os.path.join(_path, oa[len(oa) - 1])) >= max_clear:
                oa.append(others.date_format(fmt='%Y.%m.%d.%H.%M.%S') + '.log')
                setattr(self, logo_show, False)

        return os.path.join(center_name, oa[len(oa) - 1])

    def __new__(cls, *args, **kwargs):
        instance = Singleton.createObject(cls)
        return instance



import copy

from typing import List, Tuple

from aestate.exception import FieldNotExist
from aestate.dbs._mysql import tag
from aestate.util import others
from aestate.util.Log import ALog
import threading
from aestate.work.Modes import Singleton


class ParseUtil:

    def parse_main(self, *args, to_str=False, symbol='%s', left='[', right=']', **kwargs):
        """
            解析属性:
                将属性格式设置为:['`a`,','`b`,','`c`']
            :param to_str:是否转成str格式
            :param args:参数
            :param is_field:是否为表字段格式
            :param symbol:分隔符语法
            :param left:分隔符语法
            :param right:分隔符语法
            :return:
        """

        is_field = kwargs['is_field'] if 'is_field' in kwargs.keys() else False

        fields = []
        for value in args:
            value = others.conversion_types(value)
            if to_str:
                if is_field:
                    fields.append(f'{left}{symbol}{right},' % (str(value)))
                else:
                    fields.append(f'{symbol},' % (str(value)))
            else:
                fields.append(value if not ParseUtil.is_default(value) else None)
        if len(fields) != 0:
            fields[len(fields) - 1] = fields[len(fields) - 1].replace(',', '')
            field_str = ''
            if to_str:
                for field in fields:
                    field_str += field
                return field_str
            return fields
        else:
            return None

    def parse_key(self, *args, **kwargs):
        """
        解析键格式,如:
            INSERT INTO `demo` (这里的就是键) VALUES ('','','','');
        :param args:
        :return:
        """
        if 'is_field' not in kwargs.keys():
            kwargs['is_field'] = True
        fields = self.parse_main(*args, to_str=True, **kwargs)
        return fields

    def parse_value(self, *args, **kwargs):
        """
        解析值格式,如:
            INSERT INTO `demo` (`index`, `title`, `selects`, `success`) VALUES (这里的就是值);
        :param args:
        :return:
        """
        values = self.parse_main(*args, **kwargs)
        return values

    def parse_insert(self, keys, values, __table_name__, insert_str, values_str, symbol='%s',
                     sql_format='%s %s (%s)%s(%s)'):
        """
        实现此方法可自定义sql生成模式

        keys:包含了所有需要解析的字段名
        values:包含了所有需要用到的字段的值
        __table_name__:表名称
        insert_str:insert的字符串
        values_str:values字符串
        symbol:格式化方式，以`%s`作为匿名符号
        """
        fields = self.parse_key(*keys)
        values = self.parse_value(*values)
        # 分析需要几个隐藏值
        hides_value = [f'{symbol},' for i in range(len(values))]
        # 去除末尾的逗号
        end = hides_value[len(hides_value) - 1]
        hides_value[len(hides_value) - 1] = end[0: len(end) - 1]
        # 得到最后隐藏符号的字符串表达格式
        value = ''.join(hides_value)
        sql = sql_format % (
            insert_str,
            str(__table_name__), fields, values_str, value
        )

        kes = {'sql': sql}
        args = []
        [args.append(i) for i in values]
        kes['params'] = args
        return kes

    def parse_insert_pojo(self, pojo, __table_name__, insert_str, values_str):
        """
        解析插入语句

        INSERT INTO `__table_name__`(`title`,'selects') VALUE ('','')

        :param pojo:POJO对象
        :param __table_name__:表名
        :param insert_str:insert的sql方言
        :param values_str:values的sql方言
        :return:
        """
        # 得到所有的键
        ParseUtil.fieldExist(pojo, 'fields', raise_exception=True)
        # 在得到值之后解析是否为空并删除为空的值和对应的字段
        cp_value = []
        # 复制新的一张字段信息
        keys_copy = []

        keys_c, cp_v = ParseUtil.parse_pojo(pojo)
        keys_copy += keys_c
        cp_value += cp_v

        return self.parse_insert(keys_copy, cp_value, __table_name__, insert_str=insert_str,
                                 values_str=values_str)

    @staticmethod
    def parse_pojo(pojo) -> Tuple[list, list]:
        keys = pojo.fields
        # 在得到值之后解析是否为空并删除为空的值和对应的字段
        cp_value = []
        # 复制新的一张字段信息
        keys_copy = []
        values = [getattr(pojo, v) for v in keys]
        for i, j in enumerate(values):
            if j is not None and not ParseUtil.is_default(j):
                keys_copy.append(keys[i])
                cp_value.append(j)

        return keys_copy, cp_value

    @staticmethod
    def parse_obj(data: dict, instance: object) -> object:
        """
        将数据解析成对象
        注意事项:
            数据来源必须是DbUtil下查询出来的
        :param data:单行数据
        :param instance:参与解析的对象
        :return:POJO对象
        """
        # 深度复制对象
        part_obj = copy.copy(instance)
        for key, value in data.items():
            setattr(part_obj, key, value)
        return part_obj

    @staticmethod
    def find_last_id(**kwargs):
        """作者:CACode 最后编辑于2021/4/12

        遵循规则：

            内部>配置文件

        是否包含返回最后一行ID的配置

        只存在于更新操做的方法内，如：

            insert,

            update,

            delete

         Attributes:

             conf_obj:配置类
        """
        conf_obj = kwargs['config_obj']
        if 'last_id' not in kwargs.keys():
            c_dict = conf_obj.get_dict()
            if 'last_id' in c_dict.keys():
                kwargs['last_id'] = c_dict['last_id']
            else:
                kwargs['last_id'] = False
        return kwargs

    @staticmethod
    def find_print_sql(**kwargs):
        """
        遵循规则：
            内部>配置文件

        是否包含打印sql的配置

        存在于所有数据库操做

        Attributes:
             conf_obj:配置类
        """
        conf_obj = kwargs['config_obj']
        if 'print_sql' not in kwargs.keys():
            c_dict = conf_obj.get_dict()
            if 'print_sql' in c_dict.keys():
                kwargs['print_sql'] = c_dict['print_sql']
            else:
                kwargs['print_sql'] = False
        return kwargs

    @staticmethod
    def case_name(text, rep_text='_', lower=True, upper=False):
        """
        将驼峰文本改为使用指定符号分割的字符串表达形式并全部小写
        :param text:需要替换的文本
        :param rep_text:在大写文本后面追加的字符串
        :param lower:是否需要全部小写
        :param upper:是否需要全部大写
        """
        lst = []
        for index, char in enumerate(text):
            if char.isupper() and index != 0:
                lst.append(rep_text)
            lst.append(char)
        # 替换名称
        if lower:
            return "".join(lst).lower()
        elif upper:
            return "".join(lst).upper()
        else:
            return "".join(lst)

    @staticmethod
    def is_default(__val):
        """
        是否等于默认值
        """
        try:
            t_v = __val.__class__.__base__
            if t_v in [tag.Template, tag.baseTag]:
                return __val.default is None
        except SyntaxError:
            return False

    @staticmethod
    def set_field(obj, key, value):
        """
        当对象没有这个字段时，为对象设置一个字段

        为了方便提高拓展性可解耦，框架内部务必使用此

        方法或者set_field_compulsory()为操作管理类提供对象

        """
        if not hasattr(obj, key):
            setattr(obj, key, value)

    @staticmethod
    def set_field_compulsory(obj, key: str, data: dict, val: object) -> None:
        """
        如果键存在于data中，为obj插入该值，反之插入val
        """
        if key in data.keys():
            setattr(obj, key, data[key])
        else:
            setattr(obj, key, val)

    @staticmethod
    def fieldExist(obj: object, field: str, el=None, raise_exception=False) -> object:
        """
        在对象中获取一个字段的值,如果这个字段不存在,则将值设置为`el`
        """
        if isinstance(obj, dict):
            if field in obj.keys():
                return obj[field]
            else:
                if raise_exception:
                    ALog.log_error(
                        msg=f'the key of `{field}` cannot be found in the `{obj.__class__.__name__}`',
                        obj=FieldNotExist, raise_exception=True)
                else:
                    return el
        else:
            if hasattr(obj, field):
                return getattr(obj, field)
            else:
                if raise_exception:
                    raise ALog.log_error(
                        msg=f'the key of `{field}` cannot be found in the `{obj.__class__.__name__}`',
                        obj=FieldNotExist,
                        raise_exception=True)
                else:
                    return el

    @staticmethod
    def parse_pojo_many(pojo_many: list) -> List[tuple]:

        # 在得到值之后解析是否为空并删除为空的值和对应的字段
        cp_value = []
        for pojo in pojo_many:
            keys_c, cp_v = ParseUtil.parse_pojo(pojo)
            cp_value.append(tuple(cp_v))
        # 真实值
        return cp_value

    @staticmethod
    def insert_to_obj(obj, kwargs):
        for key, value in kwargs.items():
            ParseUtil.set_field_compulsory(obj=obj, key=key, data=kwargs, val=value)

    def get_pojo_sql(self, instance):
        """
        获取创建pojo对象的sql语句
        """
        fields = instance.getFields()
        print(fields)


# 所有常量以空格开头并且以空格结束
# 空格符
class Fields:
    """
    默认的数据库方言配置
    """

    _instance_lock = threading.RLock()

    @staticmethod
    def parse_field(key: str) -> str:
        return f' {key} '

    @property
    def left_subscript(self):
        """
        左角标
        """
        return '`'

    @property
    def space(self):
        """
        空格
        """
        return ' '

    @property
    def right_subscript(self):
        """
        右角标
        """
        return '`'

    @property
    def insert_str(self):
        """
        插入
        """
        return self.parse_field('INSERT INTO')

    @property
    def delete_str(self):
        """
        删除
        """
        return self.parse_field('DELETE')

    @property
    def update_str(self):
        """
        更新
        """
        return self.parse_field('UPDATE')

    @property
    def find_str(self):
        return self.parse_field('SELECT')

    @property
    def where_str(self):
        return self.parse_field('WHERE')

    @property
    def by_str(self):
        return self.parse_field('BY')

    @property
    def order_by_str(self):
        return self.parse_field('ORDER BY')

    @property
    def group_by_str(self):
        return self.parse_field('GROUP BY')

    @property
    def desc_str(self):
        return self.parse_field('DESC')

    @property
    def set_str(self):
        return self.parse_field('SET')

    @property
    def ander_str(self):
        return self.parse_field('AND')

    @property
    def limit_str(self):
        return self.parse_field('LIMIT')

    @property
    def from_str(self):
        return self.parse_field('FROM')

    @property
    def value_str(self):
        return self.parse_field('VALUE')

    @property
    def values_str(self):
        return self.parse_field('VALUES')

    @property
    def asses_str(self):
        return self.parse_field('AS')

    @property
    def left_par(self):
        return self.parse_field('(')

    @property
    def right_par(self):
        return self.parse_field(')')

    @property
    def comma(self):
        return self.parse_field(',')

    @property
    def eq(self):
        return self.parse_field('=')

    @property
    def on_str(self):
        return self.parse_field('on')

    @property
    def left_join_str(self):
        return self.parse_field('LEFT JOIN')

    @property
    def symbol(self):
        return '>> << == <= >= != - + / * %'.split(' ')

    def parse_set(self, keys):
        """
        格式化set键
        """
        keys_str = ''
        for i in keys:
            keys_str += '{}=%s{}'.format(i, self.ander_str)
        keys_str = keys_str[0:len(keys_str) - len(self.ander_str)]
        return keys_str

    def __new__(cls, *args, **kwargs):
        instance = Singleton.createObject(cls)
        return instance


class OperaBase:
    def __init__(self, instance, fields):
        self.instance = instance
        self.fields = fields

    def start(self):
        pass


class CreateModel(OperaBase):
    """
    将pojo创建为数据库的表
    """

    def start(self):
        __table_name__ = self.instance.__table_name__


class MakeModel(OperaBase):
    def start(self):
        pass



from .AopContainer import AopModelObject
from .Modes import EX_MODEL
from .Serialize import QuerySet
import os
import inspect

from .xmlhandler.nodes import ResultMapNode
from .xmlhandler.utils import AestateXml
from ..exception import TagAttributeError, TagHandlerError
from ..i18n import ExceptionI18n
from ..util.Log import ALog
from ..util.sqlOpera import TextUtil


def Table(name, msg="", **kwargs):
    """
    标注该类为一个表
    :param name:表的名称
    :param msg:表的描述
    :return:
    """

    def set_to_field(cls):
        setattr(cls, '__table_name__', name)
        setattr(cls, '__table_msg__', msg)
        for key, value in kwargs.items():
            setattr(cls, key, value)
        return cls

    return set_to_field


def Select(sql: str):
    """
    快捷的查询装饰器

    使用此装饰器,可以将大量重复代码继承到此装饰器内部实现

    使用方法:

        @Select(sql="SELECT * FROM demo_table WHERE t_id<=${不加密} AND t_msg like #{加密}")

    有两种符号可以作为sql的字段插入形式:

        ${字段}:这种方式是直接将文字插入进去,在面对sql注入时无法有效避免

        #{字段}:将字段使用%s过滤,能有效防止sql注入,但并非100%有效
            依靠的是你所使用的第三方库内置游标的:`mogrify`方法,请在使用前查看


    :param sql:执行的sql语句,需要加密的参数使用`%s`表示
    """

    def base_func(cls):
        def _wrapper_(*args, **kwargs):
            lines = list(args)
            obj = lines[0]

            # 查找参数
            sub_sql, new_args = TextUtil.replace_antlr(sql, **kwargs)

            result = obj.find_sql(sql=sub_sql, params=new_args)
            return QuerySet(obj, result)

        return _wrapper_

    return base_func


def SelectAbst():
    def mysql_rp(n, array) -> str:
        _name = array[len(array) - 1] if len(array) > 0 else ""
        rule = {
            'F': 'FROM',
            'find': "SELECT",
            'where': 'WHERE',
            'eq': "= #{%s}" % _name,
            'lt': '< #{%s}' % _name,
            'gt': '> #{%s}' % _name,
            'le': '<= #{%s}' % _name,
            'ge': '>= #{%s}' % _name,
            'in': 'in #{%s}' % _name,
            'like': 'like #{%s}' % _name,
            'all': '*',
        }
        return rule[n] if n in rule.keys() else n

    def base_func(func):
        def _wrapper_(*args, **kwargs):
            lines = list(args)
            if len(lines) == 0 and hasattr(func, 'instance') and not func.instance:
                ALog.log_error(ExceptionI18n.tt(""))
            obj = lines[0]
            _name = func.__name__.split("_")
            S = []
            for i in _name:
                d = mysql_rp(i, S)
                S.append(d if d != "FROM" else f"FROM {obj.__table_name__}")

            sql = ' '.join(S)

            # 查找参数
            sub_sql, new_args = TextUtil.replace_antlr(sql, **kwargs)

            result = obj.find_sql(sql=sub_sql, params=new_args)
            return QuerySet(obj, result)

        return _wrapper_

    return base_func


def AopModel(before=None, after=None,
             before_args=None, before_kwargs=None,
             after_args=None, after_kwargs=None):
    """

        AOP切面模式：
            依赖AopModel装饰器,再在方法上加入@AopModel即可切入编程


        优点:

            当使用@AopModel时,内部函数将会逐级调用回调函数,执行循序是:
                - func(*self.args, **self.kwargs)
                - func(*self.args)
                - func(**self.kwargs)
                - func()
            这将意味着,如果你的参数传入错误时,AopModel依旧会遵循原始方法所使用的规则,最令人大跌眼镜的使用方法就是:
<code>
                def Before(**kwargs):
                    print('Before:', kwargs)
                # 此处的Before方法未存在args参数,而使用@AopModel时却传入了args
                @AopModel(before=Before,before_args=(0,1,2), before_kwargs={'1': '1'})
                def find_title_and_selects(self, **kwargs):

                    print('function task', kwargs['uid'])

                    _r = self.orm.find().where(index="<<100").end()

                    print(_r)

                    return _r
</code>
            其中包含参数有:
                before:切入时需要执行的函数

                before_args:切入的参数
                    传入的列表或元组类型数据
                    如果是需要使用当前pojo中的内容时，传参格式为:(pojo.字段名)
                    可扩展格式，例如需要传入字典

                before_kwargs:切入的参数 -- 传入的字典数据

                after:切出前需要执行的参数

                after_args:切出的参数
                    传入的列表或元组类型数据
                    如果是需要使用当前pojo中的内容时，传参格式为:('self.字段名')
                    可扩展格式，例如需要传入字典:('self.dict.key')

                after_kwargs:切出的参数 -- 传入的字典数据


        执行流程:

            Before->original->After

        Before注意事项:

            使用该参数时，方法具有返回值概不做处理,需要返回值内容可使用`global`定义一个全局字段用于保存数值

            当无法解析或者解析失败时m将使用pass关键字忽略操作

        After注意事项:

            使用该参数时，必须搭配至少一个result=None的kwargs存在于方法的形参中,

            当original方法执行完成将把返回值固定使用result键值对注入到该函数中

            当无法解析或者解析失败时m将使用pass关键字忽略操作



        Attributes:

             before:切入时需要执行的函数

             after:切出前需要执行的参数

             before_args:切入的参数
                传入的列表或元组类型数据
                如果是需要使用当前pojo中的内容时，传参格式为:(pojo.字段名)
                可扩展格式，例如需要传入字典

             before_kwargs:切入的参数 -- 传入的字典数据

             after_args:切出的参数
                传入的列表或元组类型数据
                如果是需要使用当前pojo中的内容时，传参格式为:('self.字段名')
                可扩展格式，例如需要传入字典:('self.dict.key')

             after_kwargs:切出的参数 -- 传入的字典数据


            """
    # 得到对象组
    aop_obj = AopModelObject(before, after,
                             before_args, before_kwargs,
                             after_args, after_kwargs)

    def base_func(func):
        aop_obj.func = func

        def _wrapper_(*args, **kwargs):
            aop_obj.set_args(*args, **kwargs)
            return aop_obj.start()

        return _wrapper_

    return base_func


def ReadXml(filename):
    """读取xml"""

    def set_to_field(cls):
        sep = os.sep

        file_path = inspect.getfile(cls)
        file_path = file_path[:file_path.rfind(sep)]
        path = os.path.join(file_path, filename)

        setattr(cls, '_xml_file', path)
        xml = AestateXml.read_file(path)
        setattr(cls, 'xNode', xml)
        setattr(cls, '_xml_file_name', os.path.basename(path))
        return cls

    return set_to_field


def Item(_id, d=False):
    """
    将xml的item节点映射到当前方法,对应的id字段为xml节点的id
    """

    def replaceNextLine(sql):
        sql = str(sql).replace('\n', ' ')
        sql = str(sql).replace('  ', ' ')
        if '  ' in sql:
            return replaceNextLine(sql)
        else:
            return sql

    def base_func(cls):
        def _wrapper_(*args, **kwargs):
            lines = list(args)
            obj = lines[0]
            xml = obj.xNode

            xml_node = None
            for v in xml.children['item']:
                if 'id' in v.attrs.keys() and v.attrs['id'].text == _id:
                    xml_node = v
                    break
            if xml_node is not None:
                xml_node.params = kwargs
                result_text_node = xml_node.text(obj)
            else:
                result_text_node = None
                ALog.log_error(
                    f"`{_id}` does not exist in the xml node.file:({obj._xml_file_name})", obj=TagAttributeError,
                    raise_exception=True)
            # 美化sql
            if result_text_node is None:
                ALog.log_error(
                    f"`The node did not return any sentences.file:({obj._xml_file_name})", obj=TagHandlerError,
                    raise_exception=True)
                return None
            run_sql = replaceNextLine(result_text_node.text)
            sub_sql, params = TextUtil.replace_antlr(run_sql, **kwargs)
            # 返回值ast
            if 'resultType' in result_text_node.mark.keys():
                result = obj.execute_sql(sql=sub_sql, params=params, mode=EX_MODEL.SELECT, **obj.__dict__)
                resultTree = ResultMapNode(obj, result_text_node, result)
                if d:
                    return result
                return resultTree.apply()
            else:
                result = obj.execute_sql(sql=sub_sql, params=params, mode=EX_MODEL.UPDATE,
                                         **{'last_id': result_text_node.mark['has_last_id'], **obj.__dict__})
                return result

        return _wrapper_

    return base_func


def JsonIgnore(*fields):
    def base_func(fn):
        def _wrapper_(*args, **kwargs):
            _self = args[0]
            _self.EXEC_FUNCTION = [*fields]
            return _self

        return _wrapper_

    return base_func



import threading

from aestate.work import Modes
from aestate.work.commands import __logo__


class Banner:
    _instance_lock = threading.Lock()

    def __init__(self, status: bool = False):
        self.status = status

    @classmethod
    def show(cls):
        Modes.Singleton.println(cls, __logo__)



# -*- utf-8 -*-
import asyncio
import uuid

from aestate.work.Modes import EX_MODEL
from aestate.work.Serialize import QuerySet
from aestate.work.orm import AOrm


class RepositoryProxy:
    """
    代理仓库的操作方式所有Repository的调用都会经过这里

    这个位置是用来方便使用type对象的Pojo类行为

    通过Repository的__get__方法获得调用时的cls值,使得

    """

    @property
    def conversion(self):
        """
        将此Repository转换为ORM实体

        Return:
            ORM转换之后的实体对象
        """
        return AOrm(repository=self)

    def first(self):
        """
        获取数据库中的第一个
        """
        return self.conversion.top().end()

    def last(self):
        """
        获取最后一个参数
        """
        return self.conversion.top().desc().end()

    def find_all(self, **kwargs) -> QuerySet:
        """
        从当前数据表格中查找所有数据

        Returns:
            将所有结果封装成POJO对象集合并返回数据
        """
        # 开启任务
        self.result = self.find_field(*self.getFields(), **kwargs)
        return self.result

    def find_field(self, *args, **kwargs) -> QuerySet:
        """
        只查询指定名称的字段,如:

            SELECT user_name FROM `user`

            即可参与仅解析user_name为主的POJO对象

        :param args:需要参与解析的字段名

        :return:
            将所有结果封装成POJO对象集合并返回数据

        """
        # 设置名称
        name = str(uuid.uuid1())
        # 开启任务
        kwargs.update(
            {
                'func': self.operation.__find_by_field__,
                '__task_uuid__': name,
                't_local': self
            }
        )

        result = self.operation.start(*args, **kwargs)

        self.result = self.serializer(instance=self, base_data=result)
        return self.result

    def find_one(self, sql, **kwargs):
        """
        查找第一条数据

            可以是一条

            也可以是很多条中的第一条

        code:

            result = self.find_many(**kwargs)
            if len(result) == 0:
                return None
            else:
                return result[0]

        :param kwargs:包含所有参数:

            pojo:参照对象

            sql:处理过并加上%s的sql语句

            params:需要填充的字段

            print_sql:是否打印sql语句

        :return 返回使用find_many()的结果种第一条
        """
        kwargs['sql'] = sql
        self.result = self.find_many(**kwargs)
        if self.result is None or len(self.result) == 0:
            self.result = []
            return None
        else:
            self.result = self.result.first()
            return self.result

    def find_many(self, sql, **kwargs) -> QuerySet:
        """
        查询出多行数据

            第一个必须放置sql语句

        :param kwargs:包含所有参数:

            pojo:参照对象

            sql:处理过并加上%s的sql语句

            params:需要填充的字段

            print_sql:是否打印sql语句

        :return 将所有数据封装成POJO对象并返回

        """
        # 设置名称
        name = str(uuid.uuid1())
        kwargs['sql'] = sql
        # 开启任务
        kwargs['func'] = self.operation.__find_many__
        kwargs['__task_uuid__'] = name
        kwargs['t_local'] = self
        result = self.operation.start(**kwargs)
        self.__clear_params__()
        self.result = self.serializer(instance=self.instance, base_data=result)
        return self.result

    def find_sql(self, sql, **kwargs) -> QuerySet:
        """

        返回多个数据并用list包装:

            - 可自动化操作

            - 请尽量使用find_many(sql)操作

        :param kwargs:包含所有参数:
            sql:处理过并加上%s的sql语句
            params:需要填充的字段
            print_sql:是否打印sql语句
        """
        # kwargs['conf_obj'] = t_local.config_obj
        # 设置名称
        name = str(uuid.uuid1())
        kwargs['sql'] = sql
        # 开启任务
        kwargs['func'] = self.operation.__find_sql__
        kwargs['__task_uuid__'] = name
        kwargs['t_local'] = self
        result = self.operation.start(**kwargs)

        self.result = self.serializer(instance=self.instance, base_data=result)
        return self.result

    def update(self, key=None):
        """
        执行更新操作:
            返回受影响行数

        :param key:主键，where的参考数据
        :return:
        """
        if key is None:
            for k, v in self._fields.items():
                if hasattr(v, "primary_key") and getattr(v, 'primary_key'):
                    key = k
                    break
        name = str(uuid.uuid1())
        kwargs = {
            'pojo': self,
            'func': self.operation.__update__,
            '__task_uuid__': name,
            't_local': self,
            'key': key
        }
        # 开启任务
        self.result = self.operation.start(**kwargs)
        return self.result

    def remove(self, key=None):
        """
        执行更新操作:
            返回受影响行数

        :param key:主键，where的参考数据
        :return:
        """
        if key is None:
            for k, v in self._fields.items():
                if hasattr(v, "primary_key") and getattr(v, 'primary_key'):
                    key = k
                    break
        name = str(uuid.uuid1())
        kwargs = {
            'pojo': self,
            'func': self.operation.__remove__,
            '__task_uuid__': name,
            't_local': self,
            'key': key
        }
        # 开启任务
        self.result = self.operation.start(**kwargs)
        return self.result

    def save(self, *args, **kwargs):
        """
        将当前储存的值存入数据库
        """
        kwargs['pojo'] = self
        return self.create(*args, **kwargs)

    def create(self, pojo, **kwargs):
        """
        插入属性:
            返回受影响行数
        :param kwargs:包含所有参数:
            pojo:参照对象
            last_id:是否需要返回最后一行数据,默认False
        :return:rowcount,last_id if last_id=True
        """
        # 设置名称
        kwargs['pojo'] = pojo
        name = str(uuid.uuid1())
        # 开启任务
        kwargs['func'] = self.operation.__insert__
        kwargs['__task_uuid__'] = name
        kwargs['t_local'] = self
        self.result = self.operation.start(**kwargs)
        self.__clear_params__()
        return self.result

    def copy(self, *args, **kwargs):
        """
        复制对象进行操做

        不建议多次创建对象，建议使用 pojo.copy()来生成对象
        """
        obj = self.__class__(new=True, *args, **kwargs)
        [setattr(obj, k, v) for k, v in kwargs.items()]
        return obj

    def execute_sql(self, sql, params=None, mode=EX_MODEL.SELECT, **kwargs):
        """
        :param sql:执行的sql
        :param params:防止sql注入的参数
        :param mode:查询模式,默认使用SELECT,使用aestate.work.Modes.EX_MODEL枚举修改执行的sql类型
        :param kwargs:其他需要的参数
        """
        self.__clear_params__()
        d = self.__dict__
        d.update(kwargs)
        kwargs = d
        kwargs['print_sql'] = False if 'print_sql' not in kwargs.keys() else kwargs['print_sql'] if kwargs[
            'print_sql'] else False
        if mode is None or mode == EX_MODEL.SELECT:
            self.result = self.db_util.select(sql=sql, params=params, **kwargs)
        else:
            kwargs['last_id'] = True if 'last_id' not in kwargs.keys() else kwargs['last_id']
            self.result = self.db_util.insert(sql=sql, params=params, **kwargs)
        self.__clear_params__()
        return self.result

    def foreign_key(self, cls, key_name, field_name=None, data=None, operation=None):
        """
        根据外键来查
        :param cls:目标外键的类，注意不是对象，是类
        :param key_name:外键的id
        :param field_name:保存进去的字段名字，默认以表名命名
        :param data:使用已有的数据作为外键
        :param operation:自定义操作
        """
        child_obj = cls()
        if field_name is None:
            name = child_obj.get_tb_name()
        else:
            name = field_name
        self.datas = self.result if data is None else data
        for i in range(len(self.datas)):
            if not operation:
                data = child_obj.orm.filter(**{key_name: self.datas[i].id})
            else:
                data = operation(self.datas, i)
            self.datas[i].add_field(name, data.to_dict())

    def __clear_params__(self):
        """
        清空params参数
        :return:
        """
        if hasattr(self, 'params'):
            self.params.clear()


class RepositoryAsyncProxy(RepositoryProxy):
    """
    代理执行仓库的异步操作,详情请查看SqlOperaProxy类
    """

    def find_all_async(self, *args, **kwargs):
        async def find_all():
            pass

        return asyncio.run(find_all(*args, **kwargs))

    async def find_field_async(self, *args, **kwargs):
        return self.find_field(*args, **kwargs)

    async def find_one_async(self, *args, **kwargs):
        return self.find_one(*args, **kwargs)

    async def find_many_async(self, *args, **kwargs):
        return self.find_many(*args, **kwargs)

    async def find_sql_async(self, *args, **kwargs):
        return self.find_sql(*args, **kwargs)

    async def update_async(self, *args, **kwargs):
        return self.update(*args, **kwargs)

    async def remove_async(self, *args, **kwargs):
        return self.remove(*args, **kwargs)

    async def save_async(self, *args, **kwargs):
        return self.save(*args, **kwargs)

    async def create_async(self, pojo, **kwargs):
        return self.create(pojo, **kwargs)

    async def execute_sql_async(self, sql, params=None, mode=EX_MODEL.SELECT, **kwargs):
        return self.execute_sql(sql=sql, params=params, mode=mode, **kwargs)



import inspect

from aestate.ajson import aj
from aestate.exception import FieldNotExist
from aestate.util import pack
from aestate.util.Log import ALog

from aestate.work.Serialize import QuerySet
from aestate.work.orm import AOrm
from aestate.dbs._mysql import tag
from aestate.work import repository
from aestate.work import Banner


class Pojo(repository.Repository):
    # 是否已经初始化过对象
    __init_pojo__ = False
    # 执行的
    EXEC_FUNCTION = None

    @classmethod
    def objects(cls):
        return cls()

    def __init__(self, config_obj=None, log_conf=None, close_log=False, serializer=QuerySet, **kwargs):
        """
        初始化ORM框架
        :param config_obj:配置类
        :param log_conf:日志配置类
        :param close_log:是否关闭日志显示功能
        :param serializer:自定义序列化器,默认使用aestate.work.Serialize.QuerySet
        """
        # aestate logo
        Banner.show()

        if not hasattr(self, '__table_name__'):
            self.__table_name__ = self.__class__.__name__
        if not hasattr(self, '__table_msg__'):
            self.__table_msg__ = 'The current object has no description'

        self._fields = {}
        # 在这里将config_obj实例化
        self.serializer = serializer
        # 忽略的字段
        self.__ignore_field__ = {}
        # 添加的字段
        self.__append_field__ = {}
        # bug
        self.init_fields()
        for key, value in kwargs.items():
            self.__setattr__(key, value)
        super(Pojo, self).__init__(config_obj=config_obj,
                                   instance=self,
                                   log_conf=log_conf,
                                   close_log=close_log,
                                   serializer=serializer,
                                   **kwargs)

    def init_fields(self):
        """
        初始化字段
        最后生成的字段名`_fields`
        """
        fields = self.__dict__
        fds = {}
        for key, value in fields.items():
            # 取出这个值引用对象的父类
            sub = pack.dp_equals_base(value.__class__, tag.baseTag)
            if sub:
                if not hasattr(self, key) or getattr(self, key) is None or sub:
                    setattr(self, key, value.default)
                fds[key] = value
                fds[key].name = key

        self._fields = fds

    def get_all_using_field(self) -> dict:
        all_fields = self.getFields()
        # 合并字段
        all_fields = dict(all_fields, **self.__append_field__)
        # 删除忽略字段
        for i in self.__ignore_field__.keys():
            if i in all_fields.keys():
                del all_fields[i]
        return all_fields

    def to_json(self, bf=False):
        """
        转json字符串
        """
        return aj.parse(self.to_dict(), bf=bf)

    def to_dict(self):
        """
        将数据集转字典格式
        """
        all_fields = self.get_all_using_field()
        new_dict = {}
        for key in all_fields.keys():
            # 当字段为未填充状态时，默认定义为空
            if hasattr(self, key):
                v = getattr(self, key)
                if isinstance(v, QuerySet) or isinstance(v, Pojo):
                    new_dict[key] = v.to_dict()
                else:
                    new_dict[key] = v
            else:
                if isinstance(all_fields[key], QuerySet):
                    new_dict[key] = all_fields[key].to_dict()
                else:
                    new_dict[key] = None
        return new_dict

    def getFields(self) -> dict:
        """
        获取当前类所需要序列化的字段
        """
        return self._fields

    def add_field(self, key, default_value=None):
        """
        添加一个不会被解析忽略的字段
        """
        # UPDATE [1.0.6a2] 去除对替换的判断,将直接修改原本的值
        setattr(self, key, default_value)
        if key in self.getFields() and self.getFields().get(key).__class__ == tag.boolField:
            self.__append_field__[key] = bool(default_value)
            setattr(self, key, default_value)
        else:
            self.__append_field__[key] = default_value
            # setattr(self, key, default_value)

    def remove_field(self, key):
        """
        添加一个会被解析忽略的字段
        """
        self.__ignore_field__[key] = None

    @property
    def orm(self):
        """
        转ORM框架
        """
        return AOrm(repository=self)

    def format(self, key, name):
        """
        为指定字段的值设置别名
        """
        if 'ig' in self.getFields().keys():
            self._fields['ig'].append({
                key: name
            })
        else:
            self._fields['ig'] = []
            self.format(key, name)

    def get_tb_name(self):
        """
        获取当前pojo的表名
        """
        return self.__table_name__

    def get_database(self):
        """
        获取当前pojo的数据库连接对象
        """
        if hasattr(self, 'config_obj'):
            return self.config_obj
        ALog.log_error(
            msg='The pojo object has not been initialized yet, and no configuration items have been obtained',
            obj=FieldNotExist, LogObject=self.log_obj, raise_exception=True)

    # def __getattr__(self, item):
    #     if item in self.getFields() and self.getFields().get(item).__class__ == tag.boolField:
    #         return bool(object.__getattribute__(self, item))
    #     else:
    #         return object.__getattribute__(self, item)


class Model(Pojo):
    """
    这个只是为了满足django用户的习惯
    """
    pass



import re
import sys

from aestate.exception import BaseSqlError
from aestate.i18n import InfoI18n
from aestate.util.Log import ALog
from dbutils.pooled_db import PooledDB
from aestate.work.Cache import SqlCacheManage, CacheStatus


def parse_kwa(db, **kwargs):
    """
    解析并执行sql

    :param db:db_util对象
    :param kwargs:包含所有参数:
            last_id:是否需要返回最后一行数据,默认False
            sql:处理过并加上%s的sql语句
            params:需要填充的字段
            print_sql:是否打印sql语句
            many:是否有多个
    """

    try:
        cursor = db.cursor()
        # 是否执行多条sql
        sql = kwargs['sql']
        params = kwargs['params'] if 'params' in kwargs.keys() else None
        many_flay = 'many' in kwargs.keys() and kwargs['many']
        if ('print_sql' in kwargs.keys() and kwargs['print_sql'] is True) or (kwargs['config_obj'].print_sql is True):
            _l = sys._getframe().f_back.f_lineno
            # 输出sql
            msg = InfoI18n.tt("statement") + ' ==> ' + (f'{sql} - many=True' if many_flay else sql)
            ALog.log(obj=db, line=_l, task_name='ASQL', msg=msg,
                     LogObject=kwargs['log_obj'] if 'log_obj' in kwargs.keys() else None)
            # 输出字段
            output_params = params if params else ()
            parameters = InfoI18n.tt("parameters") + " ==> "
            for i in output_params:
                parameters += ' {}{},'.format(str(i), re.findall('<class \'(.*)\'>', str(type(i))))
            if parameters[-1] == ',':
                parameters = parameters[:-1]
            ALog.log(obj=db, line=_l, task_name='ASQL', msg=parameters,
                     LogObject=kwargs['log_obj'] if 'log_obj' in kwargs.keys() else None)

        if many_flay:
            cursor.executemany(sql,
                               tuple(params) if params else ())
        else:
            if params:
                cursor.execute(sql, tuple(params))
            else:
                cursor.execute(sql)
            # try:
            #     CACodeLog.log(obj=db, line=_l, task_name='Print Sql', msg=cursor._executed)
            # except:
            #     CACodeLog.log(obj=db, line=_l, task_name='Print Sql', msg=msg)
        return cursor
    except Exception as e:
        db.rollback()
        mysql_err = BaseSqlError(e)
        mysql_err.raise_exception()


class Db_opera(PooledDB):
    def __init__(self, *args, **kwargs):
        if 'POOL' not in kwargs or kwargs['POOL'] is None:
            self.POOL = self
        if 'POOL' in kwargs.keys():
            kwargs.pop('POOL')

        super(Db_opera, self).__init__(*args, **kwargs)

    def get_conn(self):
        """
        获取数据库连接池
        :return:
        """
        conn = self.POOL.connection()
        return conn

    def select(self, **kwargs):
        """
        查找多个
        :param kwargs:包含所有参数:
            last_id:是否需要返回最后一行数据,默认False
            sql:处理过并加上%s的sql语句
            params:需要填充的字段
            print_sql:是否打印sql语句
        :return:
        """
        db = self.get_conn()
        _l = sys._getframe().f_back.f_lineno
        try:
            cursor = db.cursor()
            sql = cursor.mogrify(kwargs['sql'], tuple(kwargs['params']) if 'params' in kwargs.keys() and kwargs[
                'params'] else ())
            # 判断和获取缓存
            scm = SqlCacheManage()
            if scm.status == CacheStatus.OPEN:
                if sql in scm:
                    values = scm.get(sql).get_value()
                    msg = InfoI18n.tt("cacheResult") + ' ==> ' + (str(len(values)) if values is not None else '0')
                    ALog.log(obj=db, line=_l, task_name='ASQL', msg=msg,
                             LogObject=kwargs['log_obj'] if 'log_obj' in kwargs.keys() else None)
                    return values
            cursor = parse_kwa(db=db, **kwargs)
            # 列名
            col = cursor.description
            data = cursor.fetchall()
            _result = []
            for data_index, data_value in enumerate(data):
                _messy = {}
                for item_index, item_value in enumerate(data_value):
                    _messy[col[item_index][0]] = item_value
                _result.append(_messy)
            # 缓存
            if scm.status == CacheStatus.OPEN:
                scm.set(sql=sql, value=_result, instance=kwargs['instance'] if 'instance' in kwargs.keys() else None)

            msg = InfoI18n.tt("selectResult") + ' ==> ' + (str(len(_result)) if _result is not None else '0')
            ALog.log(obj=db, line=_l, task_name='ASQL', msg=msg,
                     LogObject=kwargs['log_obj'] if 'log_obj' in kwargs.keys() else None)
            db.close()
            return _result
        except Exception as e:
            db.rollback()
            ALog.log_error(msg=str(e), obj=e,
                           LogObject=kwargs['log_obj'] if 'log_obj' in kwargs.keys() else None,
                           raise_exception=True)
        finally:
            db.close()

    def insert(self, many=False, **kwargs):
        """
        执行插入语句
        :param kwargs:包含所有参数:
            last_id:是否需要返回最后一行数据,默认False
            sql:处理过并加上%s的sql语句
            params:需要填充的字段
        :param many:是否为多行执行
        """
        db = self.get_conn()
        _l = sys._getframe().f_back.f_lineno
        try:
            scm = SqlCacheManage()
            scm.remove_by_instance(kwargs['instance'].get_tb_name() if 'instance' in kwargs.keys() else None)
            cursor = parse_kwa(db=db, many=many, **kwargs)
            db.commit()
            # 受影响行数
            rowcount = cursor.rowcount
            msg = InfoI18n.tt("updateResult") + ' ==> ' + str(rowcount)
            ALog.log(obj=db, line=_l, task_name='ASQL', msg=msg,
                     LogObject=kwargs['log_obj'] if 'log_obj' in kwargs.keys() else None)
            # 返回受影响行数
            if kwargs['last_id']:
                return rowcount, cursor.lastrowid
            else:
                return rowcount
        except Exception as e:
            db.rollback()
            ALog.log_error(msg=str(e), obj=e,
                           LogObject=kwargs['log_obj'] if 'log_obj' in kwargs.keys() else None,
                           raise_exception=True)
        finally:
            db.close()

    def update(self, **kwargs):
        """
        执行更新语句
        :param kwargs:包含所有参数:
            last_id:是否需要返回最后一行数据,默认False
            sql:处理过并加上%s的sql语句
            params:需要填充的字段
        """
        return self.insert(**kwargs)

    def delete(self, **kwargs):
        """
        执行删除语句
        :param kwargs:包含所有参数:
            last_id:是否需要返回最后一行数据,默认False
            sql:处理过并加上%s的sql语句
            params:需要填充的字段
        """
        self.insert(**kwargs)



import enum

from aestate.base.manage import BaseManager
from aestate.dbs import mysql
from aestate.work.Adapter import LanguageAdapter
from aestate.work.Manage import Pojo

base_dict = ['__module__', '__dict__', '__weakref__', '__doc__']


class Model(BaseManager.from_queryset(Pojo)):
    class Log(enum.Enum):
        enable = False
        path = None
        max_clear = 10

    class Config(enum.Enum):
        dbtype = None
        creator = None
        print_sql = True
        last_id = True
        adapter = LanguageAdapter

    class Database(enum.Enum):
        # 数据库地址
        host = None
        # 数据库端口
        port = 3306,
        # 数据库名
        database = None
        # 数据库用户
        user = None
        # 数据库密码
        password = None
        # 数据库创建者，如果你用的是mysql，那么这里就是pymysql，如果用的是sqlserver，那么这里就应该是pymssql
        db_type = None

    name = ""

    class Operas:
        pass



from aestate.exception import FieldNotExist
from aestate.util.Log import ALog


class LanguageAdapter:
    """
    适配器,将sql方言适配到ORM框架中,实现sql自由

    从配置表中开始配置sql方言,继承SqlLanguage类并实现抽象方法,开启

    实现当前类，在orm操作中存在自定义字段时，保证所有的操作都能够按照你所希望的那样执行
    """
    funcs = {}

    def __init__(self):
        if not hasattr(self, 'funcs'):
            self.funcs = {}
        self.__sp('like', self._like_opera)
        self.__sp('in', self._in_opera)
        self.__sp('lt', self._lt_opera)
        self.__sp('gt', self._gt_opera)
        self.__sp('le', self._le_opera)
        self.__sp('ge', self._ge_opera)
        self.__sp('eq', self._eq_opera)

    def add_lan(self, name, func):
        self.__sp(name, func)

    def _like_opera(self, instance, key, value):
        instance.args.append('`' + key + '`')
        instance.args.append(' LIKE ')
        instance.args.append('%s')
        instance.args.append(' AND ')
        instance.params.append(value)

    def _in_opera(self, instance, key, value):
        if isinstance(value, list):
            instance.args.append('`' + key + '`')
            instance.args.append(' IN ')
            value = [str(i) for i in value]
            vals = ','.join(value)
            instance.args.append(f'( {vals} )')
        else:
            ALog.log_error(
                msg='value type is not list or QuerySet object',
                obj=FieldNotExist, raise_exception=True)

    def _lt_opera(self, instance, key, value):
        instance.args.append('`' + key + '`')
        instance.args.append(' < ')
        instance.args.append('%s')
        instance.args.append(' AND ')
        instance.params.append(value)

    def _gt_opera(self, instance, key, value):
        instance.args.append('`' + key + '`')
        instance.args.append(' > ')
        instance.args.append('%s')
        instance.args.append(' AND ')
        instance.params.append(value)

    def _le_opera(self, instance, key, value):
        instance.args.append('`' + key + '`')
        instance.args.append(' <= ')
        instance.args.append('%s')
        instance.params.append(value)

    def _ge_opera(self, instance, key, value):
        instance.args.append('`' + key + '`')
        instance.args.append(' >= ')
        instance.args.append('%s')
        instance.params.append(value)

    def _eq_opera(self, instance, key, value):
        instance.args.append('`' + key + '`')
        instance.args.append(' = ')
        instance.args.append('%s')
        instance.params.append(value)

    def __sp(self, key, val):
        if key not in self.funcs.keys():
            self.funcs[key] = val



# -*- coding: utf-8 -*- #
from enum import IntEnum

class Singleton:
    """
    使用单例模式
    """

    @staticmethod
    def createFactory(cls):
        with cls._instance_lock:
            if not hasattr(cls, "__instance__"):
                cls.__instance__ = cls(cls.modules)
        return cls.__instance__

    @staticmethod
    def createObject(cls):
        with cls._instance_lock:
            if not hasattr(cls, "__instance__"):
                cls.__instance__ = object.__new__(cls)
        return cls.__instance__

    @staticmethod
    def println(cls, text):
        with cls._instance_lock:
            if not hasattr(cls, "__instance__"):
                cls.__instance__ = object.__new__(cls)
                print(text)


class Recursion:
    """
    递归
    """

    @staticmethod
    def find_key_for_dict(obj_data: dict, target: str):
        """
        在字典里重复递归,直到得出最后的值,如果查不到就返回None
        """

        def parse_list(listData: list, tag: str):
            result_list = []
            if listData is not None:
                if isinstance(listData, list):
                    for i in listData:
                        if isinstance(i, list) or isinstance(i, tuple):
                            rt = parse_list(list(i), tag)
                            if rt:
                                result_list.append(rt)
                        elif isinstance(i, dict):
                            result_list.append(parse_dict(i, tag))
                        else:
                            result_list.append(parse_obj(i, tag))

                elif isinstance(listData, tuple):
                    result_list.append(parse_list(list(listData), tag))

                elif isinstance(listData, dict):
                    result_list.append(parse_dict(listData, tag))
                else:
                    result_list.append(parse_obj(listData, tag))

            else:
                return result_list
            return result_list

        def parse_dict(dictData, tag: str):
            result_dict = []
            if dictData is not None:
                if isinstance(dictData, dict):
                    if tag in dictData.keys():
                        result_dict.append(dictData.get(tag))
                        dictData.pop(tag)
                    for index, value in dictData.items():
                        if isinstance(value, list) or isinstance(value, tuple):
                            result_dict.append(parse_list(list(value), tag))
                        elif isinstance(value, dict):
                            result_dict.append(parse_dict(value, tag))
                        else:
                            result_dict.append(parse_obj(value, tag))
                elif isinstance(dictData, list):
                    result_dict.append(parse_list(list(dictData), tag))

                else:
                    result_dict.append(parse_obj(dictData, tag))
            else:
                return None
            return result_dict

        def parse_obj(objData, tag: str):

            def load(da: dict):
                obj_item_list = []
                obj_item_dict = {}
                for i, v in da.items():
                    if i == tag:
                        result_obj.append(getattr(objData, i))
                        continue
                    if isinstance(v, list):
                        obj_item_list.append(getattr(objData, i))
                    elif isinstance(v, dict):
                        obj_item_dict[i] = v

                result_obj.append(parse_list(obj_item_list, tag))
                result_obj.append(parse_dict(obj_item_dict, tag))

            result_obj = []

            if isinstance(objData, dict):
                result_obj.append(parse_dict(objData, tag))
            elif isinstance(objData, list):
                result_obj.append(parse_list(list(objData), tag))
            else:
                if isinstance(objData, object):
                    if isinstance(objData, str):
                        return None
                    elif isinstance(objData, int):
                        return None
                    elif isinstance(objData, float):
                        return None
                    else:
                        if hasattr(objData, tag):
                            result_obj.append(getattr(objData, tag))
                            load(da=objData.__dict__)
                        else:
                            load(da=objData.__dict__)
            return result_obj

        return parse_obj(obj_data, target)


class DictTemplate(object):
    """
    字典对象模板
    """

    def __init__(self, init_data):
        self.init_data = init_data

    def add(self, key, obj):
        setattr(self, key, obj)


class DictToObject(object):
    """
    将字典转成对象，解决懒得写中括号
    """

    def __init__(self, dict_data):
        baseClass = DictTemplate(dict_data)
        self.dict_data = dict_data
        self.baseNode = baseClass
        self.verification(self.baseNode, self.dict_data)

    @staticmethod
    def conversion(dict_data: dict):
        node = DictToObject(dict_data)
        return node.baseNode

    def verification(self, node: DictTemplate, value):
        """
        验证模块
        """
        node.init_data = value
        if isinstance(value, dict):
            for key, val in value.items():
                if isinstance(val, (dict, list, tuple)):
                    val = self.verification(DictTemplate(val), val)
                node.add(key, val)

        elif isinstance(value, list):
            list_temp = []
            for val in value:
                if isinstance(val, (dict, list, tuple)):
                    val = self.verification(DictTemplate(val), val)
                list_temp.append(val)
            node.add('', list_temp)

        return node


class CaseItem:
    def __init__(self, flag, method, *args, **kwargs):
        self.flag = flag
        self.method = method
        self.args = args
        self.kwargs = kwargs

    def __str__(self):
        return str(self.flag)


class CaseOperaBase:
    def __init__(self, val, method=None, *args, **kwargs):
        self.val = val
        self.method = method
        self.args = args
        self.kwargs = kwargs

    def item(self, val, order) -> object: ...


class Case(CaseOperaBase):

    def __gt__(self, other):
        """
        左边大于右边
        """

        return self

    def __ge__(self, other):
        return int(self.val) <= int(other)

    def __lt__(self, other):
        """
        左边小于右边
        """
        return int(self.val) < int(other)

    def __le__(self, other):
        """
        左边小于等于右边
        """
        return int(self.val) >= int(other)

    def __eq__(self, other):
        """
        等于
        """
        return self.val == other

    def __ne__(self, other):
        """
        不等于
        """
        return self.val != other

    def item(self, val, order):
        order.opera[self.val] = CaseItem(self.val == val, self.method, self.args[0], **self.kwargs)
        return order


class CaseDefault(CaseOperaBase):
    def item(self, val, order):
        # order.opera[self.val] = CaseItem(True, val, *self.args, **self.kwargs)
        return order.end(self.val)


class Switch:
    """
    弥补python没有switch的缺陷
    使用教程：
            from aestate.util.others import Switch,Case,CaseDefault

            base_symbol = lambda x: x + x

            val = 3
        方式1：
            # case(选择性参数,满足条件时执行的方法,当满足条件后中间方法需要的参数)
            source = Switch(Case(val)) + \
                     Case(0, base_symbol, val) + \
                     Case(1, base_symbol, val) + \
                     Case(2, base_symbol, val) + \
                     Case(3, base_symbol, val) + \
                     Case(4, base_symbol, val) + \
                     Case(5, base_symbol, val) + \
                     CaseDefault(lambda: False)
            print(ajson.aj.parse(source, bf=True))
        方式2：
            source = Switch(Case(val)). \
            case(0, base_symbol, val). \
            case(1, base_symbol, val). \
            case(2, base_symbol, val). \
            case(3, base_symbol, val). \
            case(4, base_symbol, val). \
            case(5, base_symbol, val). \
            end(lambda: False)
        print(ajson.aj.parse(source, bf=True))
    """

    def __init__(self, val):
        self.val = val
        self.opera = {}

    def case(self, item, method, *args, **kwargs):
        if item in self.opera.keys():
            raise KeyError(f'`{item}` Already exists in the `case`')

        self.opera[item] = CaseItem(self.val == item, method, *args, **kwargs)
        return self

    def end(self, default_method, *args, **kwargs):
        """
        默认处理函数
        """

        for k, v in self.opera.items():
            if v.flag:
                return v.method(*v.args, **v.kwargs)
        return default_method(*args, **kwargs)

    def __add__(self, other):
        return other.item(self.val, self)


class EX_MODEL(IntEnum):
    SELECT = 0
    UPDATE = 1



from datetime import date, datetime

import functools

from simplejson import JSONEncoder, JSONDecoder
from decimal import Decimal

_default_encoder = JSONEncoder(
    skipkeys=False,
    ensure_ascii=True,
    check_circular=True,
    allow_nan=True,
    indent=None,
    separators=None,
    encoding='utf-8',
    default=None,
    use_decimal=True,
    namedtuple_as_object=True,
    tuple_as_array=True,
    iterable_as_array=False,
    bigint_as_string=False,
    item_sort_key=None,
    for_json=False,
    ignore_nan=False,
    int_as_string_bitcount=None,
)
_default_decoder = JSONDecoder(encoding=None, object_hook=None, object_pairs_hook=None)


class AList(list):
    def __init__(self, value: list):
        list.__init__([])
        if isinstance(value, list) and len(value) > 0:
            for item in value:
                if isinstance(item, dict):
                    self.append(ADict(item))
                elif isinstance(item, list) and len(item) > 0:
                    self.append(AList(item))
                elif isinstance(item, tuple) and len(item) > 0:
                    self.append(AList(list(item)))
                else:
                    self.append(item)


class ADict(dict):

    def __init__(self, data: dict):
        super(ADict).__init__()
        for key, value in data.items():
            if isinstance(value, dict):
                setattr(self, key, ADict(value))
                self[key] = ADict(value)
            elif isinstance(value, list) and len(value) > 0:
                setattr(self, key, AList(value))
                self[key] = AList(value)

            elif isinstance(value, tuple) and len(value) > 0:
                setattr(self, key, AList(list(value)))
                self[key] = AList(list(value))

            else:
                setattr(self, key, value)
                self[key] = value


class AJson:
    """
    Json工具
    AJson.parse(**kwargs):将任意对象解析成json字符串
    AJson.load(**kwargs):将字符串解析成字典
    """

    @staticmethod
    def dumps(obj, skipkeys=False, ensure_ascii=True, check_circular=True,
              allow_nan=True, cls=None, indent=None, separators=None,
              encoding='utf-8', default=None, use_decimal=True,
              namedtuple_as_object=True, tuple_as_array=True,
              bigint_as_string=False, sort_keys=False, item_sort_key=None,
              for_json=False, ignore_nan=False, int_as_string_bitcount=None,
              iterable_as_array=False, **kw):
        """
            转json字符串
        """
        if (not skipkeys and ensure_ascii and
                check_circular and allow_nan and
                cls is None and indent is None and separators is None and
                encoding == 'utf-8' and default is None and use_decimal
                and namedtuple_as_object and tuple_as_array and not iterable_as_array
                and not bigint_as_string and not sort_keys
                and not item_sort_key and not for_json
                and not ignore_nan and int_as_string_bitcount is None
                and not kw
        ):
            return _default_encoder.encode(obj)
        if cls is None:
            cls = JSONEncoder
        return cls(
            skipkeys=skipkeys, ensure_ascii=ensure_ascii,
            check_circular=check_circular, allow_nan=allow_nan, indent=indent,
            separators=separators, encoding=encoding, default=default,
            use_decimal=use_decimal,
            namedtuple_as_object=namedtuple_as_object,
            tuple_as_array=tuple_as_array,
            iterable_as_array=iterable_as_array,
            bigint_as_string=bigint_as_string,
            sort_keys=sort_keys,
            item_sort_key=item_sort_key,
            for_json=for_json,
            ignore_nan=ignore_nan,
            int_as_string_bitcount=int_as_string_bitcount,
            **kw).encode(obj)

    @staticmethod
    def loads(s, encoding=None, cls=None, object_hook=None, parse_float=None,
              parse_int=None, parse_constant=None, object_pairs_hook=None,
              use_decimal=False, **kw):
        """
        json转字典
        """
        if (cls is None and encoding is None and object_hook is None and
                parse_int is None and parse_float is None and
                parse_constant is None and object_pairs_hook is None
                and not use_decimal and not kw):
            return _default_decoder.decode(s)
        if cls is None:
            cls = JSONDecoder
        if object_hook is not None:
            kw['object_hook'] = object_hook
        if object_pairs_hook is not None:
            kw['object_pairs_hook'] = object_pairs_hook
        if parse_float is not None:
            kw['parse_float'] = parse_float
        if parse_int is not None:
            kw['parse_int'] = parse_int
        if parse_constant is not None:
            kw['parse_constant'] = parse_constant
        if use_decimal:
            if parse_float is not None:
                raise TypeError("use_decimal=True implies parse_float=Decimal")
            kw['parse_float'] = Decimal
        return cls(encoding=encoding, **kw).decode(s)

    @staticmethod
    def date_encoder(obj):
        if isinstance(obj, datetime):
            return obj.strftime('%Y-%m-%d %H:%M:%S')
        elif isinstance(obj, date):
            return obj.strftime('%Y-%m-%d')
        else:
            return None

    class JsonDateEncoder(JSONEncoder):
        def default(self, obj):
            return AJson.date_encoder(obj)

    class SimplejsonDateEncoder(JSONEncoder):
        def default(self, obj):
            return AJson.date_encoder(obj)

    @staticmethod
    def parse(obj, bf=False, end_load=False):
        """作者:CACode 最后编辑于2021/4/27

        将对象转换成字典格式:
            支持:
                dict
                list
                object
                list[object]
                object[list]
                object[list[object]]
                .......

        注意事项:

            bf和end_load同时只能使用一个

            当两者同时存在时,默认使用end_load功能


        :param obj:需要解析的对象
        :param bf:是否需要美化json
        :param end_load:是否需要在最后转成字典格式
        """

        def json_to_str(_obj):
            """
            json转字符串
            """
            json_f = functools.partial(
                AJson.dumps, cls=AJson.JsonDateEncoder)
            json_str = json_f(_obj)
            return json_str

        def parse_list(list_obj):
            """
            解析list数据的json
                放置了递归函数,所以不必担心解析报错或者解析不到位
            """
            obj_dicts = []
            for item in list_obj:
                # 循环集合
                if isinstance(item, list):
                    # 如果是集合则递归
                    obj_dicts.append(parse_list(item))
                elif isinstance(item, tuple):
                    # 如果是tuple元组则转成集合后递归
                    return obj_dicts.append(parse_list(list(item)))
                elif isinstance(item, dict) or isinstance(item, str):
                    # 如果是字典或者字符串,则直接交给obj_dicts填充
                    obj_dicts.append(item)
                elif isinstance(item, object):
                    # 如果是object则交给parse_obj()解析
                    obj_dicts.append(parse_obj(item))
                else:
                    obj_dicts.append(item)
            return obj_dicts

        def parse_obj(_obj) -> str:
            """
            夺命循环递递归
            """
            obj_dicts = []
            if isinstance(_obj, dict):
                _dict = _obj.__dict__
                # 如果是list,则交给parse_list(解决)
                for key, item in _dict.items():
                    obj_dicts.append({
                        key: parse_list(item)
                    })
            elif isinstance(_obj, list):
                # 如果是字典或者字符串,则直接交给obj_dicts填充
                obj_dicts.append(parse_list(_obj))
            # 由于parse_list()中有对于tuple累心的解析,所以不必担心tuple
            elif isinstance(_obj, str):
                # 如果是字典或者字符串,则直接交给obj_dicts填充
                obj_dicts = _obj
            else:
                # 如果不是list类型,则直接解析成字典
                try:
                    obj_dicts = parse_dict(_obj.__dict__)
                except AttributeError as e:
                    obj_dicts = _obj
                    # 异常警告，抛出
            return obj_dicts

        def parse_dict(_obj):
            """作者:CACode 最后编辑于2021/4/27
            解析字典格式
            """
            obj_dicts = {}
            if isinstance(_obj, dict):
                for key, value in _obj.items():
                    if isinstance(value, list):
                        obj_dicts[key] = parse_list(value)
                    elif isinstance(value, dict):
                        obj_dicts[key] = parse_dict(value)
                    else:
                        v = parse_obj(value)
                        obj_dicts[key] = v
            return obj_dicts

        # 如果他是集合并且里面包含的非字典而是object,则将对象转成字典
        if isinstance(obj, list):
            obj = parse_list(obj)
        elif isinstance(obj, dict):
            obj = parse_dict(obj)
        elif isinstance(obj, object):
            obj = parse_obj(obj)
        # 最后的解析结果
        result = json_to_str(obj)
        if end_load:
            return AJson.load(result)
        elif bf:
            return AJson.beautiful(AJson.load(result))
        return result

    @staticmethod
    def load(item):
        """作者:CACode 最后编辑于2021/4/27
        将json字符串解析成字典
        """
        if isinstance(item, list):
            _dats = []
            for i in item:
                _dats.append(AJson.load(i))
            return _dats
        elif isinstance(item, tuple):
            # 如果是tuple元组则转成集合后递归
            _dats = []
            for i in list(item):
                _dats.append(AJson.load(i))
            return _dats
        elif isinstance(item, dict):
            # 如果是字典,则直接返回
            return item
        elif isinstance(item, str):
            # 如果是字符串则解析为字典
            return AJson.loads(item)
        elif isinstance(item, object):
            # 如果是object则交给parse_obj()解析
            return item.__dict__
        else:
            return AJson.loads(item)

    @staticmethod
    def beautiful(_data):
        """作者:CACode 最后编辑于2021/4/27
        美化json
        """
        return AJson.dumps(_data, sort_keys=True, indent=4, separators=(',', ':'))

    @staticmethod
    def json_to_object(json_data):
        if isinstance(json_data, list):
            obj = AList(json_data)
        elif isinstance(json_data, tuple):
            obj = AList(list(json_data))
        elif isinstance(json_data, dict):
            obj = ADict(json_data)
        else:
            return None

        return obj



import types

from aestate.util.CompulsoryRun import Compulsory


class AopModelObject(object):
    """
        此类为AopModel提供所有操作
    """

    def __init__(self, before=None, after=None,
                 before_args=None, before_kwargs=None,
                 after_args=None, after_kwargs=None):
        # 初始化所有字段
        self.__before_func__ = before
        self.__before_args_data__ = before_args
        self.__before_kwargs_data__ = before_kwargs

        self.__after_func__ = after
        self.__after_args_data__ = after_args
        self.__after_kwargs_data__ = after_kwargs

    def set_args(self, *args, **kwargs):
        self.args = args
        self.kwargs = kwargs

    def start(self):
        """
        主操作
        """
        # self.func = args[0]
        self.init_fields()
        # wraps(self.func)(self)
        self.init_attr()

        # 解析参数需要
        # self.before_parse()
        # 执行before操作
        self.before_run()
        # 执行原始数据
        result = Compulsory.run_function(
            func=self.func, args=self.args, kwargs=self.kwargs)
        # after解析
        # self.after_parse(result)
        # after操作
        self.after_run(result)
        # 返回原始数据
        return result

    def init_fields(self):
        # 定义名称规则
        self.after = 'after'
        self.after_args = 'after_args'
        self.after_kwargs = 'after_kwargs'

        self.before = 'before'
        self.before_args = 'before_args'
        self.before_kwargs = 'before_kwargs'

        self.__after__ = '__after_func__'
        self.__after_args__ = '__after_args__'
        self.__after_kwargs__ = '__after_kwargs__'

        # 得到before参数的名称
        self.__before_name__ = self.format_name(self.before)
        self.__before_args_name__ = self.format_name(self.before_args)
        self.__before_kwargs_name__ = self.format_name(self.before_kwargs)

        # 得到after参数的名称

        self.__after_name__ = self.format_name(self.__after__)
        self.__after_args_name__ = self.format_name(self.__after_args__)
        self.__after_kwargs_name__ = self.format_name(self.__after_kwargs__)

    def __get__(self, instance, cls):
        if instance is None:
            return self
        else:
            return types.MethodType(self, instance)

    def format_name(self, name):
        """
        格式化名称字符串
        """
        return '{}{}'.format(name, self.func.__name__)

    def setters(self, i1, i2, i3, k1, v1, k2, v2, k3, v3):
        """
        批量设置
        """
        if i1 in self.__dict__.keys():
            setattr(self, v1, self.__dict__[k1])
            if i2 in self.__dict__.keys():
                setattr(self, v2, self.__dict__[k2])
            if i3 in self.__dict__.keys():
                setattr(self, v3, self.__dict__[k3])

    def init_attr(self):
        """
        初始化cls下的字段
        通过使用setters下的setter()功能批量解析是否需要before或者after操作
        """

        self.setters(
            i1=self.before,
            i2=self.before_args,
            i3=self.before_kwargs,
            k1=self.before,
            k2=self.before_args,
            k3=self.before_kwargs,
            v3=self.__before_kwargs_name__,
            v1=self.__before_name__,
            v2=self.__before_args_name__,
        )

        self.setters(
            i1=self.after,
            i2=self.after_args,
            i3=self.after_kwargs,
            k1=self.after,
            k2=self.after_args,
            k3=self.after_kwargs,
            v1=self.__after_name__,
            v2=self.__after_args_name__,
            v3=self.__after_kwargs_name__
        )

    def before_run(self):
        """
        执行before方法
        """
        if self.__before_func__ and self.__before_args_data__ and self.__before_kwargs_data__:
            self.__before_func__(*self.__before_args_data__,
                                 **self.__before_kwargs_data__)
        elif self.__before_func__ and self.__before_args_data__:
            self.__before_func__(*self.__before_args_data__)
        elif self.__before_func__ and self.__before_kwargs_data__:
            self.__before_func__(**self.__before_kwargs_data__)
        elif self.__before_func__:
            self.__before_func__()
        else:
            pass

    def after_run(self, result):
        """
        执行after方法
        """
        if self.__after_kwargs_data__ is None:
            self.__after_kwargs_data__ = {}

            self.__after_kwargs_data__.update({'result': result})
        if self.__after_func__ and self.__after_args_data__ and self.__after_kwargs_data__:
            self.__after_func__(*self.__after_args_data__,
                                **self.__after_kwargs_data__)
        elif self.__after_func__ and self.__after_args_data__:
            self.__after_func__(*self.__after_args_data__)
        elif self.__after_func__ and self.__after_kwargs_data__:
            self.__after_func__(**self.__after_kwargs_data__)
        elif self.__after_func__:
            self.__after_func__()
        else:
            pass



from aestate.work.Serialize import QuerySet
from aestate.exception import FieldNotExist
from aestate.dbs import _mysql
from aestate.work.proxy import SqlOperaProxy
from aestate.work.sql import ExecuteSql, ProxyOpera
from aestate.util.Log import ALog


class Repository(SqlOperaProxy.RepositoryAsyncProxy):
    """
    - POJO类
        - 继承该类表名此类为数据库的pojo类
        - 需要配合:@Table(name, msg, **kwargs)使用
    """

    def __init__(self, config_obj=None, instance=None, log_conf=None, close_log=False, serializer=QuerySet, **kwargs):
        """
        通过继承此类将数据表实体化

            实体化之后你可以使用像类似find_one()等操做

            可以调用conversion()方法将其转化为ORM框架常用的样式

            无需担心类型问题，无需担心datetime无法转换
        使用方法:
            #加入Table注解，并标注表名与描述，因考虑使用者后期优化问题，请务必填写MSG参数

            @Table(name="demo_table", msg="demo message")

            #继承Repository并得到相对应的半自动ORM操做
            class TestClass(Repository):
                # 初始化并super配置
                def __init__(self,**kwargs):
                    super(DemoTable, self).__init__(config_obj=ConF(), log_conf={
                        'path': "/log/",
                        'save_flag': True
                    }, **kwargs)

        初始化配置:

            aestate.util.Config.config的配置类,详见:aestate.work.Config.MysqlConfig

        Attributes:
            以下的字段均可覆盖重写

            config_obj:数据源配置类

            log_conf:日志配置工具

            log_obj:日志对象

            close_log:是否关闭日志

            serializer:序列化使用的类,默认使用aestate.work.Serialize.QuerySet

            instance:实例

            __table_name__:表名称

            operation:操作类的实现

            fields:操作的字段

            sqlFields:sql方言

        :param config_obj:配置类
        :param log_conf:日志配置类
        :param close_log:是否关闭日志显示功能
        :param serializer:自定义序列化器,默认使用aestate.work.Serialize.QuerySet
        """
        # 以下使用ParseUtil将所有参数替换为可动态修改
        if config_obj is None:
            ALog.log_error(msg="缺少配置类`config_obj`", obj=FieldNotExist, raise_exception=True)
        self.ParseUtil = config_obj
        self.ParseUtil.set_field_compulsory(
            self, key='config_obj', data=kwargs, val=config_obj)
        # 抽象类
        self.ParseUtil.set_field_compulsory(
            obj=self, data=kwargs, key='abst', val=False)
        # 当本类为抽象类时，仅设置所需要的值
        self.ParseUtil.set_field_compulsory(
            self, key='close_log', data=kwargs, val=close_log)
        # 有没有表名
        self.ParseUtil.set_field_compulsory(self, key='__table_name__', data=kwargs,
                                            val=self.__table_name__ if hasattr(self, '__table_name__') else
                                            '"__table_name__" parsing failed')
        # 参照对象
        # 能操作数据库的，但是没有值
        self.ParseUtil.set_field_compulsory(
            self, key='instance', data=kwargs, val=instance)
        # 取得字段的名称
        self.ParseUtil.set_field_compulsory(
            self, key='fields', data=kwargs, val=list(self.instance.getFields().keys()))
        # 获取sql方言配置
        self.ParseUtil.set_field_compulsory(
            self, key='sqlFields', data=self.config_obj.__dict__, val=_mysql.Fields())
        # 当当前类为抽象类时，为类取消初始化数据库配置
        # 最后的执行结果
        self.ParseUtil.set_field_compulsory(
            self, key='result', data=kwargs, val=None)
        self.ParseUtil.set_field_compulsory(self, key='log_obj', data=kwargs,
                                            val=ALog(**log_conf) if log_conf is not None else None)
        self.ParseUtil.set_field_compulsory(
            self, key='serializer', data=kwargs, val=serializer)
        if not self.abst:
            # 操作类
            self.ParseUtil.set_field_compulsory(
                self, key='operation', data=kwargs, val=ProxyOpera.DbOperation())
            # 连接池
            if hasattr(self, 'config_obj') and self.config_obj:
                self.db_util = ExecuteSql.Db_opera(
                    creator=self.ParseUtil.fieldExist(self.config_obj, 'creator', raise_exception=True),
                    POOL=None if 'POOL' not in kwargs.keys() else kwargs['POOL'],
                    **self.ParseUtil.fieldExist(self.config_obj, 'kw', raise_exception=True))
            else:
                ALog.log_error('`config_obj` is missing', AttributeError, LogObject=self.log_obj, raise_exception=True)



from concurrent.futures import ThreadPoolExecutor

pool = ThreadPoolExecutor()


class DbOperation(object):
    """
    迁移重要操作到此类
    """

    def __init__(self, *args, **kwargs):
        self.args = args
        self.kwargs = kwargs

    def start(self, *args, **kwargs):
        """
        开始任务
        Attributes:
            func:调用指定的方法
        """
        # 执行的函数体
        func = kwargs['func']
        # 线程独立
        _lock = kwargs['t_local']
        name = kwargs['__task_uuid__']
        # # 设置任务
        # _kw = aj.load(aj.parse(_lock))
        _kw = _lock.__dict__
        kwargs.update(_kw)
        _t = pool.submit(lambda x, y: func(*x, **y), args, kwargs)
        # _t = threading.Thread(target=func, args=args, kwargs=kwargs, name=name)
        # if not _lock.close_log:
        #     ALog.log(obj=_t, msg='RUNNING', task_name=name, LogObject=log_obj)
        result = _t.result()
        # 返回结果
        return result[name]

    def __find_all__(self, *args, **kwargs):
        """作者:CACode 最后编辑于2021/4/12
        任务方法
        """
        return self.__find_by_field__(*args, **kwargs)

    def __find_by_field__(self, *args, **kwargs):
        """作者:CACode 最后编辑于2021/4/12

        任务方法
        """
        fields = kwargs['config_obj'].parse_key(*args, is_field=True, left=kwargs['sqlFields'].left_subscript,
                                                right=kwargs['sqlFields'].right_subscript)
        sql_str = kwargs['sqlFields'].find_str + fields + kwargs['sqlFields'].from_str + kwargs['__table_name__']
        kwargs['sql'] = sql_str
        return self.__find_many__(**kwargs)

    def __find_many__(self, *args, **kwargs):
        """作者:CACode 最后编辑于2021/4/12

        任务方法
        """
        # kwargs['conf_obj'] = config_obj
        # kwargs = kwargs['ParseUtil'].find_print_sql(**kwargs)
        return self.__find_sql__(**kwargs)

    def __find_sql__(self, *args, **kwargs):
        """作者:CACode 最后编辑于2021/4/12

        任务方法
        """
        kwargs = kwargs['ParseUtil'].find_print_sql(**kwargs)
        _rs = kwargs['db_util'].select(**kwargs)

        result = []
        if _rs:
            for i in _rs:
                obj = kwargs['ParseUtil'].parse_obj(i, kwargs['instance'])
                result.append(obj)

        return {
            kwargs['__task_uuid__']: result
        }

    def __insert__(self, *args, **kwargs):
        """作者:CACode 最后编辑于2021/4/12
        :param pojo: pojo对象
        任务方法
        """
        kwargs = kwargs['ParseUtil'].find_print_sql(**kwargs)

        kwargs = kwargs['ParseUtil'].find_last_id(**kwargs)
        kwargs['ParseUtil'].fieldExist(kwargs, 'pojo', raise_exception=True)

        if 'many' in kwargs and kwargs['many']:
            # 多行插入 这个先取出sql语句,params无作用
            for item in kwargs['pojo']:
                filed_list = kwargs['config_obj'].parse_insert_pojo(item,
                                                                    __table_name__=kwargs['__table_name__'],
                                                                    insert_str=kwargs['sqlFields'].insert_str,
                                                                    values_str=kwargs['sqlFields'].values_str)

                if 'params' not in kwargs.keys() or not isinstance(kwargs['params'], list):
                    kwargs['params'] = []
                kwargs['sql'] = filed_list['sql']
                kwargs['params'].append(filed_list['params'])

        else:
            filed_list = kwargs['config_obj'].parse_insert_pojo(kwargs['pojo'], __table_name__=kwargs['__table_name__'],
                                                                insert_str=kwargs['sqlFields'].insert_str,
                                                                values_str=kwargs['sqlFields'].values_str)

            kwargs.update(filed_list)
        return {
            kwargs['__task_uuid__']: kwargs['db_util'].insert(**kwargs)
        }

    def __update__(self, *args, **kwargs):
        kwargs = kwargs['ParseUtil'].find_print_sql(**kwargs)

        kwargs = kwargs['ParseUtil'].find_last_id(**kwargs)
        kwargs['sql'], kwargs['params'] = kwargs['config_obj'].parse_update(kwargs['pojo'], kwargs['key'])

        return {
            kwargs['__task_uuid__']: kwargs['db_util'].update(**kwargs)
        }

    def __remove__(self, *args, **kwargs):
        kwargs = kwargs['ParseUtil'].find_print_sql(**kwargs)

        kwargs = kwargs['ParseUtil'].find_last_id(**kwargs)
        kwargs['sql'], kwargs['params'] = kwargs['config_obj'].parse_remove(kwargs['pojo'], kwargs['key'])

        return {
            kwargs['__task_uuid__']: kwargs['db_util'].update(**kwargs)
        }



# -*- coding: utf-8 -*- #
__author__ = 'CACode'

from aestate.ajson import aj

"""
此文件内包含有序列化器所有需要用到的参数
aj可使用原simplejson部分功能，内嵌simplejson，升级功能包含
- parse(obj,bf,end_load) 解析object类型
- load(obj) 生成字典
"""

__all__ = ['aj', 'QuerySet', 'PageHelp']


class QuerySet(list):
    """
    执行database operation返回的结果集对象

    此序列化器采用链表形式储存数据,递归搜索子节点

    顺序从左子树开始依次按照索引排列

    元类:
        list

    Methods:
        first():
            返回结果集对象的第一个数据

        last():
            返回结果集对象的最后一位参数

        page(size):
            按照每一页有size数量的结果分页

        to_json():
            将结果集对象转json字符串

        add_field():
            添加一个字段使得解析过程中不会被移除

        remove_field():
            删除一个字段使得解析过程中不会添加

        get():
            返回指定位置的参数

    Attribute:

        instance:实例类型模板

        base_data:基本数据

        query_item:使用已有的数据生成QuerySet对象

    """

    def __init__(self, instance=None, base_data=None, query_items=None):
        """
        初始化传入结果集并附加上base_data数据集

        instance:
            序列化的实例对象

        base_data:
            初始化数据源
        """
        list.__init__([])
        if query_items is None:
            self.__instance__ = instance
            # 合并结果集对象
            # TODO:历史遗留问题,2022/01/17修复当结果为空时不存入数据
            if base_data:
                self.extend(base_data)
        else:
            self.extend(query_items)

    def size(self):
        return len(self)

    def first(self):
        """
        取得结果集的第一位参数,如果没有就返回空列表
        """
        if len(self) > 0:
            return self[0]
        else:
            return []

    def last(self):
        """
        取得结果集的最后一位参数,如果没有就返回0
        """
        if len(self) > 0:
            return self[len(self) - 1]
        else:
            return []

    def page(self, size):
        """
        将结果集按照指定数目分割
        """
        return PageHelp.list_of_groups(init_list=self, size=size)

    def to_json(self, bf=False):
        """
        将结果集对象转json处理
        :param bf:是否需要美化sql
        """
        result = []
        for i in self:
            result.append(aj.load(i.to_json(bf=bf)))
        return aj.parse(result, bf=bf)

    def to_dict(self):
        result = []
        for i in self:
            result.append(aj.load(i.to_dict()))
        return result

    def add_field(self, key, default_value=None):
        """
        添加一个不会被解析忽略的字段
        """

        [self[i].add_field(key, default_value) for i in range(len(self))]

    def remove_field(self, key):
        """
        添加一个会被解析忽略的字段
        """
        [self[i].remove_field(key) for i in range(len(self))]

    def get(self, index):
        """
        返回指定位置的元素
        """
        return self[index]


class PageHelp(list):
    def __init__(self, init_data: list):
        list.__init__([])

        self.__dict_data__ = {}
        self.__json_data__ = ""

        self.extend(init_data)

    def to_dict(self):
        """
        节省资源
        """
        if not self.__dict_data__:
            self.__dict_data__ = aj.load(self.to_json())
        return self.__dict_data__

    def to_json(self, bf=False):
        """
        节省资源
        """
        if not self.__json_data__:
            json_str = [i.to_dict() for i in self]
            self.__json_data__ = aj.parse(json_str, bf)
        return self.__json_data__

    @classmethod
    def list_of_groups(cls, init_list, size):
        """
        将数据集按照一定数量分组并返回新数组
        """
        list.__init__([])
        lo_groups = zip(*(iter(init_list),) * size)
        end_list = [QuerySet(query_items=i) for i in lo_groups]
        count = len(init_list) % size
        end_list.append(QuerySet(
            query_items=init_list[-count:])) if count != 0 else QuerySet(query_items=end_list)
        return PageHelp(end_list)

    def get(self, index):
        return self[index]



# -*- utf-8 -*-
import importlib
import re
from abc import ABC

from aestate.exception import NotFindTemplateError, TagAttributeError, TagHandlerError, XmlParseError
from aestate.i18n import ExceptionI18n
from aestate.util.Log import ALog
from aestate.work.Serialize import QuerySet
from aestate.work.xmlhandler.XMLScriptBuilder import IfHandler
from aestate.work.xmlhandler.base import AestateNode


class AbstractNode(ABC):
    """抽象节点，所有节点的父类"""

    # 节点自身的操作

    def __init__(self, target_obj, params, aestate_xml_cls, root, value, XML_KEY, XML_IGNORE_NODES):
        self.target_obj = target_obj
        self.params = params
        self.aestate_xml_cls = aestate_xml_cls
        self.root = root
        self.node = value
        self.XML_KEY = XML_KEY
        self.XML_IGNORE_NODES = XML_IGNORE_NODES

    # 得到节点的值
    def apply(self, *args, **kwargs):
        ...

    def parseNode(self, texts: AestateNode, node):
        for root_index, root_value in enumerate(node.childNodes):
            if root_value.nodeName in self.XML_KEY.keys():
                obj = self.XML_KEY[root_value.nodeName](self.target_obj, self.params, self.aestate_xml_cls, self.root,
                                                        root_value, self.XML_KEY, self.XML_IGNORE_NODES)
                texts = obj.apply(texts=texts)
            elif root_value.nodeName in self.XML_IGNORE_NODES:
                try:
                    texts.add(node=root_value, index=root_index)
                except Exception as e:
                    ALog.log_error(
                        msg=''.join(e.args),
                        obj=e, LogObject=self.target_obj.log_obj, raise_exception=True)
            try:
                texts.extend(AestateNode(self.root, root_value))
            except Exception as e:
                ALog.log_error(
                    msg=''.join(e.args),
                    obj=e, LogObject=self.target_obj.log_obj, raise_exception=True)
        return texts


class SelectNode(AbstractNode):

    def apply(self, *args, **kwargs):
        # 取得已有的文本
        texts = kwargs['texts']
        axc_node = self.aestate_xml_cls(self.root, self.node, self.params)
        # 返回值类型
        resultType = axc_node.attrs['resultType']
        texts.mark['resultType'] = resultType.text
        return self.parseNode(texts, self.node)


class UpdateNode(AbstractNode):
    class TempTextNode:
        def __init__(self, text):
            self.text = text

    def apply(self, *args, **kwargs):
        # 取得已有的文本
        texts = kwargs['texts']
        axc_node = self.aestate_xml_cls(self.root, self.node, self.params)
        # 返回值类型
        has_last_id = axc_node.attrs['last'] if 'last' in axc_node.attrs.keys() else self.TempTextNode('True')
        texts.mark['has_last_id'] = has_last_id.text == 'True'
        return self.parseNode(texts, self.node)


class IfNode(AbstractNode):

    def conditional_test(self, text, syntax_re_text):
        # 移除空集
        syntax_using = [x for x in syntax_re_text[0] if x != '']
        if len(syntax_using) == 2 or len(syntax_using) > 3:
            ALog.log_error(
                msg=ExceptionI18n.tt('xml_syntax_error') % text,
                obj=TagHandlerError, LogObject=self.target_obj.log_obj, raise_exception=True)

        # 左边的匹配字段名,这就意味着变量必须写在左边
        initial_field = syntax_using[0]
        symbol = syntax_using[1]
        value = syntax_using[2]

        rfield = re.findall('#{(.*?)}', initial_field)

        field = rfield[0] if len(rfield) > 0 and rfield[0] in self.params.keys() else initial_field
        # 让事件器来执行
        ifhandler = IfHandler(initial_field=initial_field, field=field, params=self.params, value=value,
                              symbol=symbol)

        return ifhandler.handleNode(self.target_obj)

    def signal_conditional_test(self, text, field):
        """单个判断值对否"""
        rfield = re.findall('#{(.*?)}', field)
        if len(rfield) == 1 and rfield[0] not in self.params.keys():
            # 这里理应等于false,但是由于存在不等号,所以当没有时他应该为!false,也就是true
            return True
        return not bool(self.params[rfield])

    def apply(self, *args, **kwargs):
        texts = kwargs['texts']
        axc_node = self.aestate_xml_cls(self.root, self.node, self.params)
        if 'test' not in axc_node.attrs.keys():
            ALog.log_error(
                msg=ExceptionI18n.tt('if_tag_not_test'),
                obj=TagAttributeError, LogObject=self.target_obj.log_obj, raise_exception=True)
            return
        test_syntax = axc_node.attrs['test']
        # UPDATE: 1.0.6a2 增加!=
        tests = re.split('and|or|&&|\|\|', test_syntax.text)
        conditions = re.findall('(and|or|&&|\|\|)', test_syntax.text)
        if len(conditions) != len(tests) - 1:
            ALog.log_error(
                msg=ExceptionI18n.tt('xml_syntax_error') % test_syntax.text,
                obj=XmlParseError, LogObject=self.target_obj.log_obj, raise_exception=True)
        # UPDATE: 1.0.6a2 增加可以识别多个条件
        # 是否可以继续判断
        if_next = None
        for t in tests:
            success = False
            # 去除首尾空格寻找匹配的语法
            text = t.strip()
            # 一种是 字段-符号-值
            syntax_re_text = re.findall('(#\{.*?\})([>=|<=|==|<|>|!=]+)(.*)', text)
            # 一种是 符号-空格(可有可无)-字段
            signal_syntax_re_text = re.findall('!\s*(#\{.*?\})', text)
            if len(syntax_re_text) != 0:
                success = self.conditional_test(text, syntax_re_text)
            elif len(signal_syntax_re_text) == 1:
                success = self.signal_conditional_test(text, signal_syntax_re_text[0])
            else:
                # 缺少必要的test标签语法
                ALog.log_error(
                    msg=ExceptionI18n.tt('xml_syntax_error') % test_syntax.text,
                    obj=TagAttributeError, LogObject=self.target_obj.log_obj, raise_exception=True)

            if len(conditions) > 0:
                _and = re.search('and|&&', conditions[0])
                _or = re.search('or|\|\|', conditions[0])
                if if_next is None:
                    if_next = success
                    continue
                if _and:
                    if_next = if_next and success
                elif _or:
                    if_next = if_next or success
                else:
                    ALog.log_error(
                        msg=ExceptionI18n.tt('before_else_not_if'),
                        obj=TagHandlerError, LogObject=self.target_obj.log_obj, raise_exception=True)
                # 分割应该放在末尾,因为条件需要比判断的符号多一个索引
                conditions = conditions[1:]
            else:
                if_next = success
            # 如果已经是false
            if not if_next:
                break
        if if_next:
            texts = self.parseNode(texts, node=self.node)

        if IfHandler.checking_mark(self.node):
            # 设置为反的
            texts.mark['if_next'] = not if_next

        return texts


class ElseNode(AbstractNode):

    def apply(self, *args, **kwargs):
        texts = kwargs['texts']
        if 'if_next' not in texts.mark.keys():
            ALog.log_error(
                msg=ExceptionI18n.tt('before_else_not_if'),
                obj=TagHandlerError, LogObject=self.target_obj.log_obj, raise_exception=True)
        else:
            if_next = texts.mark['if_next']
            if not if_next:
                texts.mark.pop('if_next')
                return texts
            else:
                texts.mark.pop('if_next')
                return self.parseNode(texts, self.node)


class SwitchNode(AbstractNode):

    def apply(self, *args, **kwargs):
        texts = kwargs['texts']
        axc_node = self.aestate_xml_cls(self.root, self.node, self.params)
        field_name = axc_node.attrs['field'].text
        try:
            value = self.params[field_name]
        except KeyError as ke:
            ALog.log_error(
                msg=ExceptionI18n.tt('not_field_name') % field_name,
                obj=TagHandlerError, LogObject=self.target_obj.log_obj, raise_exception=True)
        case_nodes = axc_node.children['case']
        check_node = None
        for cn in case_nodes:
            if cn.attrs['value'].text == value:
                check_node = cn.node
                break
        if check_node is None:
            check_node = axc_node.children['default'][0].node
        texts = self.parseNode(texts, check_node)

        return texts


class IncludeNode(AbstractNode):

    def apply(self, *args, **kwargs):
        texts = kwargs['texts']
        axc_node = self.aestate_xml_cls(self.root, self.node, self.params)
        from_node_name = axc_node.attrs['from'].text
        templates = self.target_obj.xNode.children['template']
        target_template = None
        for t in templates:
            if t.attrs['id'].text == from_node_name:
                target_template = t
                break
        if target_template is None:
            ALog.log_error(
                msg=ExceptionI18n.tt('not_from_node_name') % from_node_name,
                obj=NotFindTemplateError, LogObject=self.target_obj.log_obj, raise_exception=True)
        texts = self.parseNode(texts, target_template.node)
        return texts


class ResultABC(ABC):
    @staticmethod
    def get_type(structure: dict):
        t = structure['_type'].split('.')
        cls_name = t[len(t) - 1]
        package_name = '.'.join(t[:len(t) - 1])
        package = importlib.import_module(package_name)
        _type = getattr(package, cls_name)
        return _type

    @staticmethod
    def generate(data: list, structure: dict):
        ret = []
        if not isinstance(data, list) and data is not None:
            data = [data]
        if data is not None:
            for _data_item in data:
                obj = ResultABC.get_type(structure)(abst=True)
                for field, properties in structure.items():
                    if field != '_type':
                        if isinstance(properties, dict):
                            obj.add_field(field, ResultABC.generate(_data_item, properties))
                            # obj.__append_field__
                            # setattr(obj, field, ResultABC.generate(_data_item, properties))
                        else:
                            obj.add_field(properties, _data_item[field])
                            # setattr(obj, properties, _data_item[field])
                ret.append(obj)
        else:
            ret.append(None)

        return QuerySet(query_items=ret)


class ResultMapNode(object):
    def __init__(self, target_obj, node, data):
        self.target_obj = target_obj
        self.node = node
        self.data = data

    def apply(self):
        resultMapTags = self.target_obj.xNode.children['resultMap']
        resultNode = None
        for i in resultMapTags:
            if i.attrs['id'].text == self.node.mark['resultType']:
                # 这里不需要break，因为可以重复，取最后一位
                resultNode = i

        if resultNode is None:
            ALog.log_error(
                msg=ExceptionI18n.tt('not_result_map'),
                obj=NotFindTemplateError, LogObject=self.target_obj.log_obj, raise_exception=True)
        structure = ForeignNode.apply(resultNode)
        return ResultABC.generate(self.data, structure)


class ForeignNode:
    @staticmethod
    def apply(resultNode):
        if 'type' not in resultNode.attrs.keys():
            ALog.log_error(
                msg=ExceptionI18n.tt('not_type'),
                obj=TagAttributeError, raise_exception=True)
        structure = {'_type': resultNode.attrs['type'].text}
        if 'result' in resultNode.children.keys():
            for i in resultNode.children['result']:
                structure[i.attrs['field'].text] = i.attrs['properties'].text

        if 'foreign' in resultNode.children.keys():
            for i in resultNode.children['foreign']:
                structure[i.attrs['name'].text] = ForeignNode.apply(i)
        return structure
